//==========================//
//        PainText.C        //
//  C++ Windows Paint Text  //
//==========================//

#include <windows.h>

#define APP_ICON  IDI_APPLICATION     // system icon
#define APP_MENU  NULL                // no menu
#define NUM_LINES   99

HANDLE hInst;
char   szAppTitle[] = "Painting text in Windows";
char   szAppName[]  = "PaintText";

long FAR PASCAL WndProc( HWND hwnd,   UINT msg,
                         UINT wParam, LONG lParam )
{
   static short cxChr, cxCap, cyChr, cxWin, cyWin,
                cyPos, cxPos, cyMax, cxMax;
   short        i, x, y, cyStep, cxStep, nFirst, nLast;
   HDC          hdc;
   PAINTSTRUCT  ps;
   TEXTMETRIC   tm;
   RECT         rect;
   char         szBuffer[180];

   switch( msg )
   {
      case WM_CREATE:
         hdc = GetDC( hwnd );
         GetTextMetrics( hdc, &tm );
         cxChr = tm.tmAveCharWidth;
         cxCap = ( tm.tmPitchAndFamily & 1 ? 3 : 2 )
                 * cxChr / 2;
         cyChr = tm.tmHeight + tm.tmExternalLeading;
         ReleaseDC( hwnd, hdc );
         break;

      case WM_SIZE:
         cyWin = HIWORD( lParam );
         cxWin = LOWORD( lParam );
            // setup vertical scrollbar
         cyMax = max( 0, NUM_LINES + 2 - cyWin / cyChr );
         cyPos = min( cyPos, cyMax );
         SetScrollRange( hwnd, SB_VERT, 0, cyMax, FALSE );
         SetScrollPos(   hwnd, SB_VERT,    cyPos, TRUE );
            // setup horizontal scrollbar
         cxMax = max( 0, NUM_LINES + 60 - cxWin / cxChr );
         cxPos = min( cxPos, cxMax );
         SetScrollRange( hwnd, SB_HORZ, 0, cxMax, FALSE );
         SetScrollPos(   hwnd, SB_HORZ,    cxPos, TRUE );
         break; 

      case WM_PAINT:
         hdc = BeginPaint( hwnd, &ps );
         nFirst = max( 0, cyPos + ps.rcPaint.top / cyChr );
         nLast  = min( NUM_LINES, cyPos
                       + ps.rcPaint.bottom / cyChr );
         SetTextAlign( hdc, TA_LEFT | TA_TOP );
         for( i = nFirst; i <= nLast; i++ )
         {
            x = 1 + cxChr * ( i - cxPos );
            y = 1 + cyChr * ( i - cyPos );
            TextOut( hdc, x, y, szBuffer,
                     wsprintf( szBuffer,
                            "This is line %d being displayed"
                            " at X:%d / Y:%d", i, x, y ) );
         }
         EndPaint( hwnd, &ps );
         break; 

      case WM_VSCROLL:
         switch( LOWORD( wParam ) )
         {
            case SB_LINEUP:
               cyStep = -1;
               break;

            case SB_LINEDOWN:
               cyStep =  1;
               break;

            case SB_PAGEUP:
               cyStep = min( -1, -cyWin/cyChr );
               break;

            case SB_PAGEDOWN:
               cyStep = max(  1,  cyWin/cyChr );
               break;

            case SB_THUMBPOSITION:
               cyStep = LOWORD( lParam ) - cyPos;
               break;

         //===== for experimental use only ========//
         // case SB_THUMBTRACK:
         //    cyStep = LOWORD( lParam ) - cyPos;
         //    break; 
         //========================================//

            default:
               cyStep = 0;
               break;
         }
         if( cyStep = max( -cyPos, 
                      min( cyStep, cyMax - cyPos ) ) )
         {
            cyPos += cyStep;
            ScrollWindow( hwnd, 0, -cyChr * cyStep,
                          NULL, NULL );
            SetScrollPos( hwnd, SB_VERT, cyPos, TRUE);
            UpdateWindow( hwnd );
         }
         break;

      case WM_HSCROLL:
         switch( LOWORD( wParam ) )
         {
            case SB_LINEUP:
               cxStep = -1;     
               break;

            case SB_LINEDOWN:
               cxStep =  1;     
               break;

            case SB_PAGEUP:
               cxStep = min( -1, -cxWin/cxChr );
               break;

            case SB_PAGEDOWN:
               cxStep = max(  1,  cxWin/cxChr );
               break;

            case SB_THUMBPOSITION:
               cxStep = LOWORD( lParam ) - cxPos;
               break;

         //===== for experimental use only =========//
         // case SB_THUMBTRACK:                             
         //    cxStep = LOWORD( lParam ) - cxPos;
         //    break; 
         //=========================================//

            default:
               cxStep = 0;
               break; 
         }
         if( cxStep = max( -cxPos,
                           min( cxStep, cxMax - cxPos ) ) )
         {
            cxPos += cxStep;
            ScrollWindow( hwnd, -cxChr * cxStep, 0,
                          NULL, NULL );
            SetScrollPos( hwnd, SB_HORZ, cxPos, TRUE);
         }
         break; 

      case WM_DESTROY:
         PostQuitMessage( 0 );
         break; 

      default:
         return( DefWindowProc( hwnd,   msg,
                                wParam, lParam ) );
   }
   return 0;
}

#include "template.i"

int APIENTRY WinMain( HANDLE hInstance, HANDLE hPrevInstance,
                      LPSTR  lpCmdLine, int    nCmdShow    )
{
   MSG   msg;

   if( ! hPrevInstance )              
      if( ! InitApplication( hInstance ) )
         return 0;
   if( ! InitInstance( hInstance, nCmdShow ) )
      return 0;
   while( GetMessage( &msg, NULL, 0, 0 ) )        
   {
      TranslateMessage( &msg );       
      DispatchMessage( &msg );        
   }
   return( msg.wParam );
   UNREFERENCED_PARAMETER( lpCmdLine );
}
