//===========================//
//          Editor.C         //
//  C++ Windows Editor Demo  //
//===========================//

#include <windows.h>
#include <malloc.h>

#define Buffer( x, y ) *( pBuffer + y * cxBuff + x )
#define MAX_BUFF  65535L
#define APP_MENU  NULL
// note: no icon or cursor assignments

HANDLE hInst;                  
char   szAppTitle[] = "A Simple Editor";
char   szAppName[]  = "Editor";

long APIENTRY WndProc( HWND hwnd,     UINT msg,
                       UINT wParam, LONG lParam )
{
   static char  *pBuffer = NULL;
   static int   cxChr,  cyChr,  cxWin,  cyWin,
                cxBuff, cyBuff, xCaret, yCaret;
          int   i, x, y;
   HDC          hdc;
   PAINTSTRUCT  ps;
   TEXTMETRIC   tm;

   switch( msg )
   {
      case WM_CREATE:
         hdc = GetDC( hwnd );
         SelectObject( hdc, GetStockObject( SYSTEM_FIXED_FONT ) );
         GetTextMetrics( hdc, &tm );
         cxChr = tm.tmAveCharWidth;
         cyChr = tm.tmHeight + tm.tmExternalLeading;
         ReleaseDC( hwnd, hdc );                    break;

      case WM_SIZE:
         cyWin = HIWORD( lParam );
         cxWin = LOWORD( lParam );
         cxBuff = max( 1, cxWin / cxChr );
         cyBuff = max( 1, cyWin / cyChr );
         if( pBuffer != NULL ) free( pBuffer );
         if( (LONG) cxBuff * cyBuff > MAX_BUFF ||
             ( pBuffer = malloc( cxBuff * cyBuff ) )
             == NULL )
            MessageBox( hwnd, "Insufficient memory --"
                        "Reduce window size ...",
                        "Editor Report",
                        MB_ICONEXCLAMATION | MB_OK );
         else
            for( y=0; y<cyBuff; y++ )
               for( x=0; x<cxBuff; x++ )
                  Buffer( x, y ) = ' ';
         xCaret = 0;
         yCaret = 0;
         if( hwnd == GetFocus() )
            SetCaretPos( xCaret * cxChr, yCaret * cyChr );			break;
      
      case WM_SETFOCUS:
         CreateCaret( hwnd, (HBITMAP) 1, cxChr, cyChr );
         SetCaretPos( xCaret * cxChr, yCaret * cyChr );
         ShowCaret( hwnd );													break;

      case WM_KILLFOCUS:  HideCaret( hwnd );
                          DestroyCaret();									break;

      case WM_KEYDOWN:
         switch( LOWORD( wParam ) )
         {
            case VK_HOME:   xCaret = 0;									break;
            case VK_END:    xCaret = cxBuff - 1;						break;
            case VK_PRIOR:  yCaret = 0;									break;
            case VK_NEXT:   yCaret = cyBuff - 1;						break;
            case VK_LEFT:   xCaret = max( 0, xCaret-1 );				break;
            case VK_RIGHT:	 xCaret = min( cxBuff-1, xCaret+1 );	break;
            case VK_UP:		 yCaret = max( 0, yCaret-1 );				break;
            case VK_DOWN:	 yCaret = min( cyBuff-1, yCaret+1 );	break;

            case VK_DELETE:
               for( x=xCaret; x<cxBuff-1; x++ )
                  Buffer( x, yCaret ) = Buffer( x+1, yCaret );
               Buffer( cxBuff-1, yCaret ) = ' ';
               HideCaret( hwnd );
               hdc = GetDC( hwnd );
               SelectObject( hdc,
                  GetStockObject( SYSTEM_FIXED_FONT ) );
               TextOut( hdc, xCaret * cxChr, yCaret * cyChr,
                        &Buffer( xCaret, yCaret ), cxBuff - xCaret );
               ShowCaret( hwnd );
               ReleaseDC( hwnd, hdc );
               break;
         }
         SetCaretPos( xCaret * cxChr, yCaret * cyChr );				break;

      case WM_CHAR:
         for( i=0; i<(int)LOWORD(lParam); i++ )
         {
            switch( LOWORD( wParam ) )
            {
               case '\b':         // backspace
                  if( xCaret > 0 )
                  {  xCaret--;
                     SendMessage( hwnd, WM_KEYDOWN,
                                  VK_DELETE, 1L );
                  }																break;

               case '\t':         // tab
                  do
                     SendMessage( hwnd, WM_CHAR, ' ', 1L);
                  while( xCaret % 8 != 0 );								break;

               case '\r':         // carriage return
                  xCaret = 0;     // falls through to '\n'

               case '\n':         // line feed            
                  if( ++yCaret == cyBuff ) yCaret = 0;				break;

               case '\x1B':       // escape
                  if( MessageBox( hwnd,
                     "Reset text buffer?",
                     "Editor Query",
                     MB_ICONEXCLAMATION | MB_OKCANCEL |
                     MB_DEFBUTTON2 ) == IDOK )
                  {
                     for( y=0; y<cyBuff; y++ )
                        for( x=0; x<cxBuff; x++ )
                           Buffer( x, y ) = ' ';
                     xCaret = 0;
                     yCaret = 0;
                     InvalidateRect( hwnd, NULL, FALSE );			
						}																break;

               default:           // all other chars
                  Buffer( xCaret, yCaret ) = 
                     (char) LOWORD( wParam );
                  HideCaret( hwnd );
                  hdc = GetDC( hwnd );
                  SelectObject( hdc, GetStockObject( SYSTEM_FIXED_FONT ));
                  TextOut( hdc, xCaret * cxChr, yCaret * cyChr,
                           &Buffer( xCaret, yCaret ), 1 );
                  ShowCaret( hwnd );
                  ReleaseDC( hwnd, hdc );
                  if( ++xCaret == cxBuff )
                  {
                     xCaret = 0;
                     if( ++yCaret == cyBuff ) yCaret = 0;
                  }																break;
         }  }
         SetCaretPos( xCaret * cxChr, yCaret * cyChr );
         break;

      case WM_PAINT:
         hdc = BeginPaint( hwnd, &ps );
         SelectObject( hdc, GetStockObject( SYSTEM_FIXED_FONT ) );
         for( y=0; y<cyBuff; y++ )
            TextOut( hdc, 0, y*cyChr, &Buffer( 0, y ), cxBuff );
         EndPaint( hwnd, &ps );												break;

      case WM_DESTROY:  PostQuitMessage( 0 );							break;

      default: return( DefWindowProc( hwnd, msg, wParam, lParam ) );
   }
   return FALSE;
}

#include "template.i"

int APIENTRY WinMain( HANDLE hInstance, HANDLE hPrevInstance,
                      LPSTR  lpCmdLine, int    nCmdShow    )
{
   MSG   msg;

   if( ! hPrevInstance )
      if( ! InitApplication( hInstance ) )
         return( FALSE );
   if( ! InitInstance( hInstance, nCmdShow ) )
      return (FALSE);
   while( GetMessage( &msg, NULL, 0, 0 ) )
   {
      TranslateMessage( &msg );
      DispatchMessage( &msg );
   }
   return( msg.wParam );
   UNREFERENCED_PARAMETER( lpCmdLine );
}
