//==========================================================//
//  FileView == demonstrates Resource Toolkit interactions  //
//==========================================================//

#include <windows.h>
#include <stdio.h>
#include <string.h>
#include "fileview.h"

#define  wFileAttr  0x4010    // file attribute flag

HANDLE    hInst,              // application hInstance
          hAccel,             // accellerator handle
          OrgFont,            // original font handle
          NewFont;            // new (fixed) font handle
HWND      hWndMain;           // hwnd of main window
LOGFONT   FontLog;            // font structure
int       ChrX, ChrY, ChrV,   // character size
          WndX, WndY,         // client window size
          vScrlMax, hScrlMax, // scroll ranges
          vScrlPos, hScrlPos, // current scroll positions
          vInc, hInc,         // scroll increments
          FilLines,           // number of lines in file
          WndWidth,           // display format width
          WndHeight,          // max lines on screen
          iFileType;          // file type
char      szAppName[10],      // application name
          szFileSpec[64],     // initial file spec
          szFileExt[5],       // default extension
          szFileName[64],     // file name
          szFilePath[64],     // file path
          szFName[64];        // file to display
WORD      wStatus;            // status of search
FILE     *hFil;               // open file handle
LONG      FilSz,              // size of file
          HDisplay;           // display width (LONG)

char     *szFileType[15] =
             { ".BMP", ".C",   ".COM", ".CUR", ".DAT",
               ".DBS", ".DLG", ".DLL", ".EXE", ".H",
               ".PAL", ".RC",  ".RES", ".TXT", ".*" };

//================ function prototypes =================//

int PASCAL WinMain( HANDLE hInstance,
                    HANDLE hPrevInstance,
                    LPSTR  lpszCmdLine,
                    int    cmdShow );

//================ exported procedures =================//

       BOOL APIENTRY About(    HWND hDlg,   UINT msg,
                               UINT wParam, LONG lParam );
       long APIENTRY WndProc(  HWND hwnd,   UINT msg,
                               UINT wParam, LONG lParam);
extern BOOL APIENTRY FileOpen( HWND hDlg,   UINT msg,
                               UINT wParam, LONG lParam );
extern BOOL APIENTRY FileType( HWND hDlg,   UINT msg,
                               UINT wParam, LONG lParam );

//================ end declarations ====================//

void ErrorMsg( char * Message )
{
   MessageBeep(0);
   MessageBox( GetFocus(), Message, "Error",
               MB_ICONASTERISK | MB_OK );
   return;
}

//======================================================//
//  CloseFV: used to clean up application instance      //
//======================================================//

void CloseFV() { }

//======================================================//
// SetScrlRange: sets vertical and horizontal scroll    //
//               ranges based on number of lines in     //
//               file and width of display              //
//      returns: nothing, sets global values            //
//======================================================//

void SetScrlRange( HWND hwnd )
{
   vScrlMax = max( 0, FilLines + 1 - (int)(WndY/ChrY) );
   vScrlPos = min( vScrlPos, vScrlMax );
   hScrlMax = max( 0, WndWidth - WndX / ChrX );
   hScrlPos = min( hScrlPos, hScrlMax );
   SetScrollRange( hwnd, SB_VERT, 0, vScrlMax, FALSE );
   SetScrollPos(   hwnd, SB_VERT,    vScrlPos, TRUE  );
   SetScrollRange( hwnd, SB_HORZ, 0, hScrlMax, FALSE );
   SetScrollPos(   hwnd, SB_HORZ,    hScrlPos, TRUE  );
   WndHeight = min( FilLines, WndY / ChrY );
}

//======================================================//
// FormatLine: formats 8/16 bytes per line for display, // 
//             beginning with file offset, hex values   //
//             grouped by fours and ASCII text at end.  //
//    returns: pointer to szBuff                        //
//======================================================//

char *FormatLine( char *szBuff, LPSTR lpMem,
                  int   iLen,   char *LnOfs )
{
   int            i, j;
   unsigned char  chr, szTemp1[80], szTemp2[80];

   *szBuff = 0;
   i = j = 0;
   wsprintf( szTemp1, "%04X  ", LnOfs );    // line offset
   lstrcpy( szBuff, szTemp1 );              // add to buff
   for( i=0; i<min(iLen,HDisplay); i++ )
   {
      chr = *lpMem++;
      if( (i+1) % 4 )                   // write hex in
         wsprintf( szTemp1, "%02X ",  chr );   
      else                              // columns of four
         wsprintf( szTemp1, "%02X  ", chr );   
      lstrcat( szBuff, szTemp1 );       // add hex to buff
      if( ( chr >= 0x20 && chr <= 0x7E ) ||
          ( chr >= 0xA0 && chr <= 0xFE ) )
         szTemp2[i] = chr;
      else
         szTemp2[i] = '.';
   }
   szTemp2[i] = '\0';            
   for( j=i; j<HDisplay; j++ )
   {
      if( (j+1) % 4 ) lstrcat( szBuff, "   " );
                 else lstrcat( szBuff, "    " );
      lstrcat( szTemp2, " " );
   }
   wsprintf( szTemp1, " |%s|", szTemp2 );  // format ASCII
   lstrcat( szBuff, szTemp1 );             // add ASCII
   return( szBuff );
}

//======================================================//
// PaintFile: if FilLines is greater than zero, reads   //
//            and displays some portion of the file     //
//            according to the size and scroll position //
//    return: nothing                                   //
//      note: file is opened, read and closed. Files    //
//            should not remain open over multiple      //
//            Window messages.                          //
//======================================================//

void PaintFile( HWND hwnd )
{
   PAINTSTRUCT  ps;
   HDC          hdc;
   LONG         lFilOfs;
   int          i, iFilSz, iTxtSz, hFile, lPos;
   char         szBuff[128];
   HANDLE       hBuff;
   LPSTR        lpStr1, lpStr2;

   BeginPaint( hwnd, (LPPAINTSTRUCT) &ps );
   hdc = ps.hdc;
   SelectObject( hdc, NewFont );   // use fixed font in DC
   if( FilLines )
   {
      if( ( hFile = _lopen( szFName, OF_READ ) ) != -1 )
      {                           // get buffer to hold data
         iFilSz = WndHeight * HDisplay;
         hBuff =
            GlobalAlloc( GMEM_MOVEABLE, (DWORD) iFilSz );
         if( hBuff )
         {                     // lock buffer for pointer
            lpStr1 = GlobalLock( hBuff );
            if( lpStr1 )
            {                  // get file offset to start
               lFilOfs = (LONG) vScrlPos * HDisplay;
               if( _llseek( hFile, lFilOfs, 0 ) != -1L )
               {
                  iTxtSz =
                     _lread( hFile, lpStr1, iFilSz );
                  if( iTxtSz != -1 )
                  {
                     lPos = 0;
                     lpStr2 = lpStr1;
                     for( i = vScrlPos;
                          i < (vScrlPos+WndHeight); i++ )
                     {
                        if( iTxtSz > HDisplay )
                           iFilSz = HDisplay;
                        else
                           iFilSz = iTxtSz;
                        iTxtSz -= iFilSz;
                        FormatLine( szBuff, lpStr2,
                           iFilSz,
                           (char *)(i * HDisplay) );
                        TextOut( hdc,
                                 ChrX * (-hScrlPos + 0),
                                 ChrY * lPos++,
                                 szBuff,
                                 lstrlen( szBuff ) );
                        lpStr2 += HDisplay;
               }  }  }
               GlobalUnlock( hBuff );
            }
            GlobalFree( hBuff );
         }
         _lclose( hFile );
   }  }
   EndPaint( hwnd, (LPPAINTSTRUCT) &ps );
}

//======================================================//
// CallFileOpen: invokes FileOpen for filename to open  //
//   parameters: szFileSpecIn  - initial file path      //
//               szFileExtIn   - initial file extension //
//  (returned)   szFilePathOut - selected file path     //
//  (returned)   szFileNameOut - selected file name     //
//       return: TRUE if file selected, FALSE if not    //
//======================================================//

int CallFileOpen( HANDLE hInst,  HWND  hwnd,
           char  *szFileSpecIn,  char *szFileExtIn,
           char  *szFilePathOut, char *szFileNameOut )
{
   int       iReturn;

   lstrcpy( szFileSpec, szFileSpecIn );   // save file spec
   lstrcpy( szFileExt,  szFileExtIn  );
   iReturn = DialogBox( hInst, "OPENFILE",
                        hwnd,  FileOpen ); 
   lstrcpy( szFilePathOut, szFilePath ); // return filepath
   lstrcat( szFilePathOut, "\\" ); 
   lstrcpy( szFileNameOut, szFileName ); // return filename
   return( iReturn );
}

//======================================================//
//   About: About dialog procedure                      //
//======================================================//

#ifdef __BORLANDC__
#pragma argsused   // prevents unused parameter message //
#endif

BOOL APIENTRY About( HWND hDlg,   UINT msg,
                     UINT wParam, LONG lParam )
{
   switch( msg )
   {
       case WM_INITDIALOG: return( TRUE );
       case WM_COMMAND:
          switch( wParam )
          {
             case IDOK: EndDialog( hDlg, TRUE );
                        return( TRUE );
               default: return( TRUE );
   }      }
   return( FALSE );
}

//======================================================//
//  WndProc: handles application messages               //
//======================================================//

long APIENTRY WndProc( HWND hwnd,   UINT msg,
                       UINT wParam, LONG lParam )
{
   static char szTmpFilePath[64];
   static char szTmpFileExt[5];
   static char szTmpFileSpec[64];
   static char szTmpFileName[64];
   static OFSTRUCT FileBuff;
   char     szBuff[128]; 
   DWORD    FilSz, FilSzHigh;
   int      hFil;

   switch( msg )
   {
      case WM_CREATE:
         GetCurrentDirectory( sizeof( szTmpFilePath ),
                              szTmpFilePath );
         lstrcat( szTmpFilePath, "\\" );
         lstrcpy( szTmpFileExt,  szFileType[iFileType] );
         lstrcpy( szTmpFileName, "" );
         HDisplay = 16L;
         break;

      case WM_COMMAND:
         switch( LOWORD( wParam ) )
         {
            case IDM_ABOUT:
               DialogBox( hInst, "ABOUT",
                          hwnd,  About );
               break;

            case IDM_TYPE:
               DialogBox( hInst, "FILETYPE",
                          hwnd,  FileType );
               break;

            case IDM_OPEN:      // set initial search path
               lstrcpy( szTmpFileSpec, szTmpFilePath );
               lstrcat( szTmpFileSpec, "*" );
               lstrcat( szTmpFileSpec,
                        szFileType[iFileType] );
               if( CallFileOpen( hInst, hwnd,
                      szTmpFileSpec, szFileType[iFileType],
                      szTmpFilePath, szTmpFileName ) )
               {
                  lstrcpy( szFName, szTmpFilePath );
                  lstrcat( szFName, "\\" );
                  lstrcat( szFName, szTmpFileName );
                  wsprintf( szBuff, "%s - %s",
                            szAppName, szFName );
                  SetWindowText( hwnd, szBuff );
                  hFil = OpenFile( szFName, &FileBuff,
                        OF_CANCEL | OF_PROMPT | OF_READ );
                  if( hFil != -1 )        // determine file size
                  {
                     FilSz = GetFileSize( (HANDLE) hFil, &FilSzHigh );
                     if( FilSz == 0xFFFFFFFF )
                     {
                        ErrorMsg( (char *) IDS_ERROR1 );
                        break;
                     }
                     if( FilSzHigh > 0 )
                     {
                        ErrorMsg( (char *) IDS_ERROR2 );
                        break;
                     }
                     FilLines =
                        (int)(( FilSz + HDisplay - 1L )
                              / HDisplay );
                     FormatLine( szBuff, szFName,
                                 HDisplay, NULL );
                                    // get display width
                     WndWidth = lstrlen( szBuff );
                     vScrlPos = hScrlPos = 0;
                     _lclose( hFil );  
                  }
                  else              // if file open failed
                  {
                     SetWindowText( hwnd, "FileView" );
                     FilLines = WndWidth = 0;
                  }
                  SetScrlRange( hwnd );
                  InvalidateRect( hwnd, NULL, TRUE );
                  UpdateWindow( hwnd );
               }  break;

            case IDM_QUIT:
               PostMessage( hwnd, WM_CLOSE, 0, 0L );
               break;

            default:
               return( DefWindowProc( hwnd,   msg,
                                      lParam, wParam ) );

         }  break;

      case WM_SIZE:
         if( lParam )
         {
            WndY = HIWORD( lParam );   // get vert size
            WndX = LOWORD( lParam );   // get horiz size
            WndY = ( WndY / ChrV + 1 ) * ChrV;
            if( ( WndX / ChrX ) < 60 ) HDisplay =  8L;
                                 else  HDisplay = 16L;
            FilLines = (int)( ( FilSz + HDisplay-1L )
                              / HDisplay );
            SetScrlRange( hwnd );      // set scroll range
            lParam = MAKELONG( WndX, WndY );
         }  break;

      case WM_VSCROLL:
         switch( LOWORD( wParam ) )
         {
            case SB_TOP:
               vInc = -vScrlPos;                    break;
            case SB_BOTTOM:
               vInc =  vScrlMax - vScrlPos;         break;
            case SB_LINEUP:
               vInc = -1;                           break;
            case SB_LINEDOWN:
               vInc =  1;                           break;
            case SB_PAGEUP:
               vInc = -max( 1, WndY / ChrY );       break;
            case SB_PAGEDOWN:
               vInc =  max( 1, WndY / ChrY );       break;
            case SB_THUMBPOSITION:
               vInc = LOWORD(lParam) - vScrlPos;    break;
            case SB_THUMBTRACK:
               vInc = LOWORD(lParam) - vScrlPos;    break;
            default:
               vInc = 0;
         }
         vInc = max( -vScrlPos,
                     min( vInc, vScrlMax - vScrlPos ) );
         if( vInc )
         {
            vScrlPos += vInc;
            ScrollWindow( hwnd, 0, -ChrY * vInc, 0L, 0L );
            SetScrollPos( hwnd, SB_VERT, vScrlPos, TRUE );
            UpdateWindow( hwnd );
         }  break;

      case WM_HSCROLL:
         switch( LOWORD( wParam ) )
         {
            case SB_TOP:
               hInc = -hScrlPos;                    break;
            case SB_BOTTOM:
               hInc =  hScrlMax - hScrlPos;         break;
            case SB_LINEUP:
               hInc = -1;                           break;
            case SB_LINEDOWN:
               hInc =  1;                           break;
            case SB_PAGEUP:
               hInc = -8;                           break;
            case SB_PAGEDOWN:
               hInc =  8;                           break;
            case SB_THUMBPOSITION:
               hInc = LOWORD(lParam) - hScrlPos;    break;
            case SB_THUMBTRACK:
               hInc = LOWORD(lParam) - hScrlPos;    break;
            default:
               hInc = 0;
         }
         hInc = max( -hScrlPos,
                     min( hInc, hScrlMax - hScrlPos ) );
         if( hInc )
         {
            hScrlPos += hInc;
            ScrollWindow( hwnd, -ChrX * hInc, 0, 0L, 0L );
            SetScrollPos( hwnd, SB_HORZ, hScrlPos, TRUE );
            UpdateWindow( hwnd );
         }  break;

      case WM_KEYDOWN:      // translate keydown messages
         switch( wParam )   //      to scrollbar messages
         {
            case VK_HOME:
               SendMessage( hwnd, WM_HSCROLL,
                            SB_TOP,      0L );
               SendMessage( hwnd, WM_VSCROLL,
                            SB_TOP,      0L );      break;
            case VK_END:
               SendMessage( hwnd, WM_HSCROLL,
                            SB_BOTTOM,   0L );
               SendMessage( hwnd, WM_VSCROLL,
                            SB_BOTTOM,   0L );      break;
            case VK_PRIOR:
               SendMessage( hwnd, WM_VSCROLL,
                            SB_PAGEUP,   0L );      break;
            case VK_NEXT:
               SendMessage( hwnd, WM_VSCROLL,
                            SB_PAGEDOWN, 0L );      break;
            case VK_UP:
               SendMessage( hwnd, WM_VSCROLL,
                            SB_LINEUP,   0L );      break;
            case VK_DOWN:
               SendMessage( hwnd, WM_VSCROLL,
                            SB_LINEDOWN, 0L );      break;
            case VK_LEFT:
               SendMessage( hwnd, WM_HSCROLL,
                            SB_PAGEUP,   0L );      break;
            case VK_RIGHT:
               SendMessage( hwnd, WM_HSCROLL,
                            SB_PAGEDOWN, 0L );      break;
         }  break;

      case WM_PAINT:
         PaintFile( hwnd );                         break;

      case WM_CLOSE:
         DestroyWindow( hwnd );                     break;

      case WM_DESTROY:
         CloseFV();   PostQuitMessage(0);           break;

      case WM_QUERYENDSESSION:
         CloseFV();  return( (LONG) TRUE );

      default:
         return( DefWindowProc( hwnd,   msg,
                                wParam, lParam ) );
   }
   return FALSE;
}

//======================================================//
// FileOpen - get the name of a file to open            //
//======================================================//

BOOL APIENTRY FileOpen( HWND hDlg,   UINT msg,
                        UINT wParam, LONG lParam )
{
   static char   OrgPath[64];
   char          cLastChar;
   int           nLen;

   switch (msg)
   {
      case WM_INITDIALOG:
         GetCurrentDirectory( sizeof(OrgPath),
                              OrgPath );  
         SendDlgItemMessage( hDlg, IDD_FNAME,
                             EM_LIMITTEXT, 80, 0L );
//============================================================
//=======   restricts filenames to 8.3 format   ==============
//       SendDlgItemMessage( hDlg, IDD_FNAME,
//                           EM_LIMITTEXT, 12, 0L );
//============================================================
         // fill list box using starting file spec
         DlgDirList( hDlg, szFileSpec,
                     IDD_FLIST, IDD_FPATH, wFileAttr );
         // show init filespec
         SetDlgItemText( hDlg, IDD_FNAME, szFileSpec );
         return( TRUE );

      case WM_COMMAND:
         switch( LOWORD( wParam ) )
         {
            case IDM_TYPE:
               if( DialogBox( hInst, "FILETYPE",
                              hDlg,  FileType ) )
               {
                  lstrcpy( szFileSpec, "*" );
                  lstrcat( szFileSpec,
                           szFileType[iFileType] );
                  DlgDirList( hDlg, szFileSpec, IDD_FLIST,
                              IDD_FPATH, wFileAttr );
                  SetDlgItemText( hDlg, IDD_FNAME,
                                  szFileSpec );
                  MessageBeep(0);
               }
               return( TRUE );

            case IDD_FLIST:
               switch( HIWORD( wParam ) )
               {
                  case LBN_SELCHANGE:
                     if( DlgDirSelectEx( hDlg, szFileName,
                            sizeof(szFileName), IDD_FLIST ) )
                        lstrcat( szFileName, szFileSpec );
                     SetDlgItemText( hDlg, IDD_FNAME,
                                     szFileName );
                     return( TRUE );

                  case LBN_DBLCLK:
                     if( DlgDirSelectEx( hDlg, szFileName,
                            sizeof(szFileName), IDD_FLIST ) )
                     {
                        lstrcat( szFileName, szFileSpec );
                        DlgDirList( hDlg, szFileName,
                                    IDD_FLIST, IDD_FPATH,
                                    wFileAttr );
                        SetDlgItemText( hDlg, IDD_FNAME,
                                        szFileSpec );
                     }
                     else
                     {
                        SetDlgItemText( hDlg, IDD_FNAME,
                                        szFileName );
                        SendMessage( hDlg, WM_COMMAND,
                                     IDOK, 0L );
                     }
                     return( TRUE );

               }  break;

            case IDD_FNAME:
               if( HIWORD( wParam ) == EN_CHANGE )
               {
                  EnableWindow( GetDlgItem( hDlg, IDOK ),
                     (BOOL) SendMessage( (HWND) lParam,
                               WM_GETTEXTLENGTH, 0, 0L ));
                  return( TRUE );
               }
               else
                  return( FALSE );

            case IDOK:
               GetDlgItemText( hDlg, IDD_FNAME,
                               szFileName, 80 );
               nLen  = lstrlen( szFileName );
               cLastChar = *AnsiPrev( szFileName,
                                      szFileName + nLen );
               if( ( cLastChar != '\\' ) &&
                   ( cLastChar != ':' ) )
                    lstrcat( szFileName, "\\" );
               else lstrcat( szFileName, szFileSpec );
               if( strchr( szFileName, '*' ) ||
                   strchr( szFileName, '?' ) )
               {
                  if( DlgDirList( hDlg,      szFileName,
                                  IDD_FLIST, IDD_FPATH,
                                  wFileAttr ) )
                  {
                     lstrcpy( szFileSpec, szFileName );
                     SetDlgItemText( hDlg, IDD_FNAME, szFileSpec );
                  }
                  else MessageBeep(0);
                  return( TRUE );
               }
               if( DlgDirList( hDlg,      szFileName,
                               IDD_FLIST, IDD_FPATH,
                               wFileAttr ) )
               {
                  lstrcpy( szFileSpec, szFileName );
                  SetDlgItemText( hDlg, IDD_FNAME, szFileSpec );
                  return( TRUE );
               }
               szFileName[nLen] = '\0';
               GetDlgItemText( hDlg, IDD_FPATH, szFilePath, 80 );
               CharUpperBuff( szFilePath, lstrlen( szFilePath ) );
               CharUpperBuff( szFileName, lstrlen( szFileName ) );
               SetCurrentDirectory( OrgPath );          // reset directory
               EndDialog( hDlg, TRUE );                 // end dialog box
               return( TRUE );

            case IDCANCEL:
               SetCurrentDirectory( OrgPath );          // reset directory
               EndDialog( hDlg, FALSE );                // end dialog box
               return( TRUE );

         }  break;
   }
   return FALSE;
}

//======================================================//
// FileType - get file type to open                     //
//======================================================//

#ifdef __BORLANDC__
#pragma argsused   // prevents unused parameter message //
#endif

BOOL APIENTRY FileType( HWND hDlg,   UINT msg,
                        UINT wParam, LONG lParam )
{
   int  iInitType;

   switch (msg)
   {
      case WM_INITDIALOG:
         CheckRadioButton( hDlg, IDD_BMP, IDD_ANY,
                           IDD_BMP+iFileType );
         iInitType = iFileType;
         return( TRUE );              // done, msg handled

      case WM_COMMAND:
         switch( LOWORD( wParam ) )
         {
            case IDD_BMP:
            case IDD_C:
            case IDD_COM:
            case IDD_CUR:
            case IDD_DAT:
            case IDD_DBS:
            case IDD_DLG:
            case IDD_DLL:
            case IDD_EXE:
            case IDD_H:
            case IDD_PAL:
            case IDD_RC:
            case IDD_RES:
            case IDD_TXT:
            case IDD_ANY:
               // set file type
               iFileType = wParam - IDD_BMP;
               // set extension
               lstrcpy( szFileExt, szFileType[iFileType] );
               return( TRUE );        // done, msg handled

            case IDOK:
               EndDialog( hDlg, TRUE );  // end dialog box
               break;

            case IDCANCEL:
               iFileType = iInitType;
               EndDialog( hDlg, FALSE ); // end dialog box
               break;

            default:  return( FALSE );  // msg not handled

         }  break;

      default: return( FALSE );         // msg not handled
   }
   return( TRUE );                    // done, msg handled
}

//======================================================//
// WinMain - ViewFile main                              //
//======================================================//

#ifdef __BORLANDC__
#pragma argsused   // prevents unused parameter message //
#endif

int PASCAL WinMain( HANDLE hInstance,
                    HANDLE hPrevInstance,
                    LPSTR  lpszCmdLine,
                    int    cmdShow )
{
   MSG        msg;
   HDC        hdc;
   WNDCLASS   wc;
   TEXTMETRIC tm;

   if( !hPrevInstance ) // if no prev instance, t'is first
   {
      LoadString( hInstance, IDS_NAME,
                  (LPSTR) szAppName, 10 );
      wc.lpszClassName = szAppName;
      wc.hInstance     = hInstance;
      wc.lpfnWndProc   = WndProc;
      wc.hCursor       = LoadCursor( NULL, IDC_ARROW );
      wc.hIcon         = LoadIcon( hInstance, szAppName );
      wc.lpszMenuName  = (LPSTR) szAppName;
      wc.hbrBackground = GetStockObject( WHITE_BRUSH );
      wc.style         = CS_HREDRAW | CS_VREDRAW;
      wc.cbClsExtra    = 0;
      wc.cbWndExtra    = 0;
      RegisterClass(&wc);
   }
   iFileType = IDD_ANY - IDD_BMP;     // initial file type
                                  // set default extension
   lstrcpy( szFileExt, szFileType[iFileType] );
   hInst = hInstance;     // global: save for window procs
   hWndMain =
      CreateWindow( szAppName, szAppName,
         WS_OVERLAPPEDWINDOW | WS_HSCROLL | WS_VSCROLL,
         CW_USEDEFAULT,  0, CW_USEDEFAULT,  0,
         NULL, NULL, hInstance, NULL );
   hdc = GetDC( hWndMain );
   FontLog.lfHeight         =  6;
   FontLog.lfWidth          =  6;
   FontLog.lfEscapement     =  0;
   FontLog.lfOrientation    =  0;
   FontLog.lfWeight         =  FW_NORMAL;
   FontLog.lfItalic         =  FALSE;
   FontLog.lfUnderline      =  FALSE;
   FontLog.lfStrikeOut      =  FALSE;
   FontLog.lfCharSet        =  ANSI_CHARSET;
   FontLog.lfOutPrecision   =  OUT_DEFAULT_PRECIS;
   FontLog.lfClipPrecision  =  CLIP_DEFAULT_PRECIS;
   FontLog.lfQuality        =  DEFAULT_QUALITY;
   FontLog.lfPitchAndFamily =  FIXED_PITCH | FF_DONTCARE;
   lstrcpy( FontLog.lfFaceName, "System" );
   NewFont = CreateFontIndirect( (LPLOGFONT) &FontLog );
   OrgFont = SelectObject( hdc, NewFont );
   GetTextMetrics( hdc, &tm );
   ChrX = tm.tmAveCharWidth;
   ChrY = tm.tmHeight + tm.tmExternalLeading;
   ChrV = tm.tmHeight;
   ReleaseDC( hWndMain, hdc );
   ShowWindow( hWndMain, cmdShow );
   UpdateWindow( hWndMain );
   hAccel = LoadAccelerators( hInst, "FILEVIEW" );
   while( GetMessage( &msg, NULL, 0, 0 ) )
   {
      if( !TranslateAccelerator( hWndMain, hAccel, &msg ) )
      {
         TranslateMessage( &msg );
         DispatchMessage( &msg );
		}  
	}
   return(msg.wParam);
}
