//  UNWIND_B
//  Shows how an exception may be handled by an except block
//  several frames behind in the stack.  Uses a filter function.

#include <windows.h>
#include <stdio.h>
#include <float.h>
#include <conio.h>

// Function prototypes.
//

void MiddleProc( void );                // intermediate function
void InnerProc( void );                 // causes various exceptions
LONG MyFilter( DWORD dwCode, int *y );  // exception filter function

//  MAIN
//  Program entry point.

int main ( )
{
    puts( "Starting program" );

    __try
    {
        MiddleProc();
    }

    // This exception handler responds if anything lower
    // down in the program generates an exception without
    // providing a handler.

    __except (EXCEPTION_EXECUTE_HANDLER)
    {
        puts( "Unwind encountered an unknown error" );
    }

    puts( "Program ending normally" );
	getch();
    return( 0 );
}

//  MIDDLE PROC
//  Calls the function that raises exceptions.

void MiddleProc ( void )
{
    puts( "Starting MiddleProc" );

    __try
    {
        puts( "In try block of MiddleProc." );
        InnerProc();
    }
    __except ( ( GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION )
		       ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH )
    {
        puts( "MiddleProc recognized an access violation." );
    }

    puts( "MiddleProc ending normally" );
    return;
}

//  INNER PROC
//  Causes exceptions.

void InnerProc ( void )
{
    char *	szText = NULL;
    int		i2 = 0;
    float	f1 = FLT_MAX;
    UINT	uFPMask;

    puts( "Starting InnerProc." );

    /* turn floating point errors into exceptions */
    uFPMask = _controlfp( 0, 0 );
    uFPMask &= ~EM_OVERFLOW;
    _controlfp( uFPMask, MCW_EM );

    __try
    {
        // This block can produce any of three different
        // exceptions.  Comment out some lines to test
        // how the program handles each exception.

//        puts( "Forcing an integer divide-by-zero error." );
//        i1 = i1 / i2;                   // handled in InnerProc

//        puts( "Forcing an access violation." );
//        lstrcpy( szText, "Hello" );   // handled in MiddleProc

        puts( "Forcing a floating point overflow." );
        f1 = f1 * f1;                   // handled in main
    }
    __except (MyFilter( GetExceptionCode(), &i2 ))
    {};

    puts( "InnerProc ending normally." );
    return;
}

// MY FILTER
// Filter function for the exception handler in InnerProc 

LONG MyFilter (
    DWORD dwCode,
    int *i )
{
    if (dwCode == EXCEPTION_INT_DIVIDE_BY_ZERO)
    {
        puts( "MyFilter fixed divide-by-zero exception" );
        *i = 1;                                 // modify offending variable
        return( EXCEPTION_CONTINUE_EXECUTION ); // try that instruction again
    }
    return( EXCEPTION_CONTINUE_SEARCH );
}
