/*--------------------------------------------------------------------

	FINDADDR.CPP

	Demonstrates using Win32 Winsock "database" functions to perform IP
   address and name lookup. Takes a single parameter, as follows:

      FINDADDR some.address.com

   Attempts to resolve domain name into its assigned IP address using
   DNS lookup. If successful, then attempts to reverse-lookup the
   domain name using the retrieved IP address. This name may be
   different than the name supplied via the command-line, and depends
   on how the IP address has been mapped.

   WINSOCK FUNCTIONS USED:

      WSAStartup              Initializes specific winsock version.
      gethostbyname           Attempts to lookup host name from IP #.
      inet_ntoa               Converts address to dotted format.
      inet_addr               Converts dotted address to 32-bits.
      gethostbyaddr           Attempts to lookup host IP# from name.
      WSACleanup              Deallocates internal winsock resources.


   NOTES:

     - This program supports both Windows 98 and NT 4.0, and requires
       Microsoft TCP/IP networking (or another TCP/IP stack) to be
       installed and configured prior to use. (You will also need to
       have a TCP/IP connection established or else have auto-dialing
       enabled).

		written/updated by Jim Blaney, 1998
		for Windows 98 Developers Handbook


  -------------------------------------------------------------------*/

#define WIN32_LEAN_AND_MEAN
#include <winsock2.h>
#include <stdlib.h>
#include <stdio.h>

// Insist on at least Winsock version 1.1 (you may use 2.0).
const	VERSION_MAJOR	=  1; 
const	VERSION_MINOR	=  1;

void ShowUsage(void)
{
	printf("usage: FINDADDR some.address.com\n");
	exit(0);
}

// Takes string IP address and attempts to look up numeric IP address.
int main(int argc, char *argv[])
{
   WSADATA     WSData;
   LPHOSTENT   lpHostEntry;
   DWORD       dwIPAddress;
   LPSTR       szIPAddress;
	
   // Check for valid # of command-line arguments.
   if ( argc != 2 ) ShowUsage(); 
	
      // Attempt to intialize WinSock (1.1 or later).
   if ( WSAStartup(MAKEWORD(VERSION_MAJOR, VERSION_MINOR), &WSData) ) 
   {
      printf("Cannot find Winsock (v%d.%d or later)!\n", 
         VERSION_MAJOR, VERSION_MINOR);
      return(1);
   }
		
   lpHostEntry = gethostbyname( argv[1] );

   if ( lpHostEntry == NULL ) 
      printf("Unable to find %s!\n", argv[1]);
   else {
      // Success. Convert address into dotted-notation (string).
      szIPAddress = inet_ntoa( *(LPIN_ADDR) *(lpHostEntry->h_addr_list) );
      
      // Display resolved address.
      printf("Found! IP address is %s\n", szIPAddress );
		
      // Now, let's try reverse lookup.
      dwIPAddress = inet_addr( szIPAddress ); // First, convert to 32-bit value.
      lpHostEntry = gethostbyaddr( (LPSTR) &dwIPAddress, sizeof(dwIPAddress), AF_INET);
		
      // Display reverse-resolved address (or error message).
      if ( lpHostEntry == NULL ) 
         printf("Hm... Unable to reverse lookup %s\n", szIPAddress);
      else 
         printf("Successful reverse lookup of %s yields %s\n", 
            szIPAddress, lpHostEntry->h_name );
      }

   // Clean up Winsock resources and terminate.
   WSACleanup();
   return(0);
}
	


