//=====================//
//       Modes.C       //
//  C++ Windows Modes  //
//=====================//

#include <windows.h>
#include <string.h>
#include <stdlib.h>
#include "modes.h"

#define APP_ICON  "MODES"
#define APP_MENU  "MODES"
// note: no cursor assignment

HANDLE	hInst;
int		XViewAspect = 0,    YViewAspect = 0,
        XWinAspect  = 1000, YWinAspect  = 2000;
char    szAppName[] = "MODES";
char    szAppTitle[] = "Hardware Device Capabilites";

BOOL APIENTRY AspectProc( HWND hDlg,    UINT msg,
                          UINT wParam, LONG lParam )
{
	char szAspect[10];

	switch( msg ) 
	{
		case WM_INITDIALOG:
			wsprintf( szAspect, "%d", XViewAspect );
			SendDlgItemMessage( hDlg, IDD_XVIEWASPECT, EM_LIMITTEXT, 10, 0L );
			SetDlgItemText( hDlg, IDD_XVIEWASPECT, szAspect );
			wsprintf( szAspect, "%d", YViewAspect );
			SendDlgItemMessage( hDlg, IDD_YVIEWASPECT, EM_LIMITTEXT, 10, 0L );
			SetDlgItemText( hDlg, IDD_YVIEWASPECT, szAspect );
			wsprintf( szAspect, "%d", XWinAspect );
			SendDlgItemMessage( hDlg, IDD_XWINASPECT, EM_LIMITTEXT, 10, 0L );
			SetDlgItemText( hDlg, IDD_XWINASPECT, szAspect );
			wsprintf( szAspect, "%d", YWinAspect );
			SendDlgItemMessage( hDlg, IDD_YWINASPECT, EM_LIMITTEXT, 10, 0L );
			SetDlgItemText( hDlg, IDD_YWINASPECT, szAspect );
			break;

		case WM_COMMAND:
			switch( LOWORD( wParam ) )
			{
				case IDOK:
					GetDlgItemText( hDlg, IDD_XVIEWASPECT, szAspect, 10 );
					XViewAspect = atoi( szAspect );
					GetDlgItemText( hDlg, IDD_YVIEWASPECT, szAspect, 10 );
					YViewAspect = atoi( szAspect );
					GetDlgItemText( hDlg, IDD_XWINASPECT, szAspect, 10 );
					XWinAspect = atoi( szAspect );
					GetDlgItemText( hDlg, IDD_YWINASPECT, szAspect, 10 );
					YWinAspect = atoi( szAspect );
					EndDialog( hDlg, TRUE );
					break;

				case IDCANCEL:
					EndDialog( hDlg, FALSE );
					break;
			}
			break;
	}
	return 0;
}


long APIENTRY WndProc( HWND hwnd,     UINT msg,
                       UINT wParam, LONG lParam )
{
	static int	nCurOrg  = IDM_UPLEFT,
				nCurMode = IDM_TEXT;
	static char *szCurMode[] = { "Text", "Low Metric", "High Metric", 
								 "Low English", "High English", "TWIPS",
								 "Isotropic", "Anisotropic" },
                *szCurOrg[]  = { "Upper Left", "Lower Left", "Centered" };
	int			nBuff, xLeft, xRight, xCenter,
				yTop, yBottom, yCenter, OrgDC,
				cxChr, cyChr;
	char		szBuff[64];
	HDC			hdc;
	HMENU		hMenu;
	RECT		rect;
	PAINTSTRUCT	ps;
	TEXTMETRIC	tm;
	FARPROC		lpProc;

	switch( msg )
	{
		case WM_CREATE:
			wsprintf( szBuff, "%s  :  %s",
					  szCurMode[nCurMode-IDM_TEXT],
					  szCurOrg[nCurOrg-IDM_UPLEFT] );
			SetWindowText( hwnd, szBuff );
			break;

		case WM_COMMAND:
			hMenu = GetMenu( hwnd );
			CheckMenuItem( hMenu, nCurMode, MF_UNCHECKED );
			CheckMenuItem( hMenu, nCurOrg,  MF_UNCHECKED );
			switch( LOWORD( wParam ) )
			{
				case IDM_ASPECT:
					lpProc = MakeProcInstance( AspectProc, hInst );
					DialogBox( hInst, "ASPECT", hwnd, lpProc );
					FreeProcInstance( lpProc );
					break;

				case IDM_TEXT:
					EnableMenuItem( hMenu, IDM_ASPECT, MF_GRAYED );
					DrawMenuBar( hwnd );
					nCurMode = wParam;
					nCurOrg  = IDM_UPLEFT;		// optimum for text mode //
					break;

				case IDM_TWIPS:
				case IDM_LOMETRIC:	case IDM_HIMETRIC:
				case IDM_LOENGLISH:	case IDM_HIENGLISH:
					EnableMenuItem( hMenu, IDM_ASPECT, MF_GRAYED );
					DrawMenuBar( hwnd );
					nCurMode = wParam;
					nCurOrg  = IDM_DNLEFT;		// optimum for metric modes //
					break;

				case IDM_ISOTROPIC:
				case IDM_ANISOTROPIC:
					EnableMenuItem( hMenu, IDM_ASPECT, MF_ENABLED );
					DrawMenuBar( hwnd );
					nCurMode = wParam;
					nCurOrg  = IDM_CENTER;
					break;

				case IDM_UPLEFT:	nCurOrg = IDM_UPLEFT;  break;
				case IDM_DNLEFT:	nCurOrg = IDM_DNLEFT;  break;
				case IDM_CENTER:	nCurOrg = IDM_CENTER;  break;

				default:
				return( DefWindowProc( hwnd, msg, wParam, lParam ) );
			}
			InvalidateRect( hwnd, NULL, TRUE );
			CheckMenuItem( hMenu, nCurMode, MF_CHECKED );
			CheckMenuItem( hMenu, nCurOrg,  MF_CHECKED );
			wsprintf( szBuff, "%s  :  %s",
					  szCurMode[nCurMode-IDM_TEXT],
					  szCurOrg[nCurOrg-IDM_UPLEFT] );
			SetWindowText( hwnd, szBuff );
			break;

		case WM_DEVMODECHANGE:
			InvalidateRect( hwnd, NULL, TRUE );
			break;

		case WM_SIZE:
			InvalidateRect( hwnd, NULL, TRUE );
			break;

		case WM_PAINT:
			hdc = BeginPaint( hwnd, &ps );
			SelectObject( hdc, GetStockObject( SYSTEM_FIXED_FONT ) );
			GetTextMetrics( hdc, &tm );
			cxChr = tm.tmAveCharWidth;
			cyChr = tm.tmHeight + tm.tmExternalLeading;
			OrgDC = SaveDC( hdc );

			SetMapMode( hdc, nCurMode - IDM_TEXT + 1 );
			SetWindowOrgEx( hdc, 0, 0, NULL );
			switch( nCurMode )
			{
				case IDM_ISOTROPIC:
					SetWindowExtEx( hdc, XWinAspect, YWinAspect, NULL );
					if( XViewAspect && YViewAspect )
						SetViewportExtEx( hdc, XViewAspect, YViewAspect, NULL );
					else 
						SetViewportExtEx( hdc, 1000, 1000, NULL );
					break;

				case IDM_ANISOTROPIC:
					SetWindowExtEx( hdc, XWinAspect, YWinAspect, NULL );
					if( XViewAspect && YViewAspect )
						SetViewportExtEx( hdc, XViewAspect, YViewAspect, NULL );
					else 
						SetViewportExtEx( hdc, XWinAspect, YWinAspect, NULL );
					break;
			}
			GetClientRect( hwnd, &rect );
			DPtoLP( hdc, (LPPOINT) &rect, 2 );
			switch( nCurOrg )
			{
				case IDM_UPLEFT:
					SetWindowOrgEx( hdc, 0, 0, NULL );
					break;

				case IDM_DNLEFT:
					SetWindowOrgEx( hdc, 0, -rect.bottom, NULL );
					break;

				case IDM_CENTER:
					SetWindowOrgEx( hdc, - ( rect.left + rect.right ) / 2,
									- ( rect.top + rect.bottom ) / 2, NULL );
					break;
			}
			GetClientRect( hwnd, &rect );
			DPtoLP( hdc, (LPPOINT) &rect, 2 );
			xLeft   = rect.left;
			xRight  = rect.right;
			yTop    = rect.top;
			yBottom = rect.bottom;
			xCenter = ( xRight + xLeft ) / 2;
			yCenter = ( yTop + yBottom ) / 2;

			RestoreDC( hdc, OrgDC );
			GetClientRect( hwnd, &rect );
			DPtoLP( hdc, (LPPOINT) &rect, 2 );
			nBuff = wsprintf( szBuff, "UpLeft: (%d,%d)", xLeft, yTop);
			TextOut( hdc, rect.left, rect.top, szBuff, nBuff );

			nBuff = wsprintf( szBuff, "DnLeft: (%d,%d)", xLeft, yBottom );
			TextOut( hdc, rect.left, rect.bottom-cyChr, szBuff, nBuff );

			nBuff = wsprintf( szBuff, "Center: (%d,%d)", xCenter, yCenter );
			TextOut( hdc, (rect.right-rect.left-nBuff*cxChr)/2,
					 (rect.bottom-rect.top)/2, szBuff, nBuff );

			nBuff = wsprintf( szBuff, "UpRight: (%d,%d)", xRight, yTop);
			TextOut( hdc, rect.right-(nBuff*cxChr), rect.top, szBuff, nBuff );

			nBuff = wsprintf( szBuff, "DnRight: (%d,%d)", xRight, yBottom );
			TextOut( hdc, rect.right-(nBuff*cxChr),
					 rect.bottom-cyChr, szBuff, nBuff );

			EndPaint( hwnd, &ps );
			break;

		case WM_DESTROY:  PostQuitMessage(0);  break;

		default:	return( DefWindowProc( hwnd, msg, wParam, lParam ) );
	}
	return 0;
}

#include "template.i"

int APIENTRY WinMain( HANDLE hInstance,	HANDLE hPrevInstance,
                      LPSTR  lpCmdLine,	int    nCmdShow    )
{
	MSG   msg;

	if( ! hPrevInstance )
		if( ! InitApplication( hInstance ) )
			return( FALSE );
	if( ! InitInstance( hInstance, nCmdShow ) )
		return (FALSE);
	while( GetMessage( &msg, NULL, 0, 0 ) )
	{
		TranslateMessage( &msg );
		DispatchMessage( &msg );
	}
	return( msg.wParam );
	UNREFERENCED_PARAMETER( lpCmdLine );
}
