//=======================//
//       PenDraw.C       //
//  C++ Windows Drawing  //
//=======================//

#include <windows.h>
#include <string.h>
#include "pendraw.h"

#define APP_ICON  "PENDRAW"
#define APP_MENU  "PENDRAW"
// note: no cursor assignment

HANDLE hInst;
char   szAppTitle[] = "PenDraw",
       szAppName[]  = "PENDRAW",
     * ColorLabels[8] =
		{	"Black", "Blue",    "Green",  "Cyan",
			"Red",   "Magenta", "Yellow", "White"  };
COLORREF lpColor[8] =
	{	RGB(   0,   0,   0 ),	RGB(   0,   0, 255 ),	//	Black,  Blue
		RGB(   0, 255,   0 ),	RGB(   0, 255, 255 ),	//	Green,  Cyan
		RGB( 255,   0,   0 ),	RGB( 255,   0, 255 ),	//	Red,    Magenta
		RGB( 255, 255,   0 ),	RGB( 255, 255, 255 ) };	//	Yellow, White
UINT   DrawModes[16] =
	{	R2_NOP,			R2_NOT,			R2_COPYPEN,		R2_NOTCOPYPEN,
		R2_MASKPEN,		R2_MASKNOTPEN,	R2_MASKPENNOT,	R2_NOTMASKPEN,
		R2_MERGEPEN,	R2_MERGENOTPEN,	R2_MERGEPENNOT,	R2_NOTMERGEPEN,
		R2_XORPEN,		R2_NOTXORPEN,	R2_BLACK,		R2_WHITE   };

struct tagLabel
{
	char * szModes;
	int	 nLen;
}	Labels;

char * szModes[16] = 
{	" <----- R2_NOP",			" <----- R2_NOT",         
	" <----- R2_COPYPEN",		" <----- R2_NOTCOPYPEN",	
	" <----- R2_MASKPEN",		" <----- R2_MASKNOTPEN",
	" <----- R2_MASKPENNOT",	" <----- R2_NOTMASKPEN",  
	" <----- R2_MERGEPEN",		" <----- R2_MERGENOTPEN",	
	" <----- R2_MERGEPENNOT",	" <----- R2_NOTMERGEPEN",
	" <----- R2_XORPEN",		" <----- R2_NOTXORPEN",
	" <----- R2_BLACK",			" <----- R2_WHITE"   };

long APIENTRY WndProc( HWND hwnd,   UINT msg,
                       UINT wParam, LONG lParam )
{
	static  int	nColor = IDM_BLACK;
	HDC			hdc;
	HMENU		hMenu;
	HBRUSH		hBrush;
	HPEN		hPen;
	PAINTSTRUCT	ps;
	RECT		rect;
	int			i, j, HUnits = 60, VUnits = 18;
	char		WinTitle[20];

	switch( msg )
	{
		case WM_COMMAND:
			hMenu = GetMenu( hwnd );
			CheckMenuItem( hMenu, nColor, MF_UNCHECKED );
			nColor = wParam;
			CheckMenuItem( hMenu, nColor, MF_CHECKED );
			InvalidateRect( hwnd, NULL, FALSE );
			wsprintf( WinTitle, "PenDraw: %s", ColorLabels[ nColor-IDM_BLACK] );
			SetWindowText( hwnd, WinTitle ); 
			break;

		case WM_SIZE:
			InvalidateRect( hwnd, NULL, TRUE );
			break;

		case WM_PAINT:
			hdc = BeginPaint( hwnd, &ps );
			GetClientRect( hwnd, &rect );
			for( j=0; j<16; j++ )
				TextOut( hdc, rect.right-190, 
				   (int) ( ( j + .80 ) * ( ( rect.bottom - rect.top ) / VUnits ) ), 
						 szModes[j], strlen( szModes[j] ) );
			SetMapMode( hdc, MM_ANISOTROPIC );
			SetViewportExtEx( hdc, rect.right-140, rect.bottom, NULL );
			SetWindowExtEx( hdc, HUnits, VUnits, NULL );
				//=== draw gray background bars ===	(white thru black)
			for( i=0; i<5; i++ )
			{
				SetRect( &rect, i*5, 0, i*5+5, VUnits );
				FillRect( hdc, &rect, GetStockObject( i ) );
			}
			i*=5;
				//=== draw color background bars === (blue thru yellow)
			for( j=1; j<7; j++ )
			{
				hBrush = CreateSolidBrush( lpColor[ j ] );
				SetRect( &rect, j*5+i-5, 0, j*5+i, VUnits );
				FillRect( hdc, &rect, hBrush );
			}
			hPen = CreatePen( PS_SOLID, 2, lpColor[nColor-IDM_BLACK] );
			for( j=0; j<16; j++ )
			{
				SetROP2( hdc, DrawModes[j] );
				SelectObject( hdc, hPen );
				MoveToEx( hdc, 1, j+1, NULL );
				LineTo( hdc, HUnits-1, j+1 );
			}
			EndPaint( hwnd, &ps );
			DeleteObject( hPen );
			break;

		case WM_DESTROY:  PostQuitMessage(0);  break;

		default:	return( DefWindowProc( hwnd, msg, wParam, lParam ) );
	}
	return 0;
}

#include "template.i"

int APIENTRY WinMain( HANDLE hInstance, HANDLE hPrevInstance,
                      LPSTR  lpCmdLine, int    nCmdShow    )
{
	MSG   msg;

	if( ! hPrevInstance )
		if( ! InitApplication( hInstance ) )
			return( FALSE );
	if( ! InitInstance( hInstance, nCmdShow ) )
		return (FALSE);
	while( GetMessage( &msg, NULL, 0, 0 ) )
	{
		TranslateMessage( &msg );
		DispatchMessage( &msg );
	}
	return( msg.wParam );
	UNREFERENCED_PARAMETER( lpCmdLine );
}
