//=======================================================================//
// InitApplication is called -- at initialization time -- if and only if //
// no other instances of this application are currently running.         //
//                                                                       //
// In this example, the window class is initialized by assigning values  //
// to the WNDCLASS structure before calling RegisterClass(). Because all //
// instances of this application use the same window class, this task is //
// executed only when the first application instance is initialized.     //
//=======================================================================//

BOOL InitApplication( HANDLE hInstance )	// identifies current instance
{
	WNDCLASS  wc;

	wc.style = (UINT) NULL;				// class style(s)
	wc.lpfnWndProc = (WNDPROC) WndProc;	// function retrieving messages
										// for windows of this class 
	wc.cbClsExtra    = 0;				// no extra class data      
	wc.cbWndExtra    = 0;				// no extra window data     
	wc.hInstance     = hInstance;		// application owning class 
#ifdef APP_ICON							// application has icon
	wc.hIcon         = LoadIcon( hInstance, APP_ICON );
#else									// application has no icon
	wc.hIcon         = LoadIcon( NULL, IDI_APPLICATION );
#endif
#ifdef APP_CURSOR						// assign application cursor
	wc.hCursor       = LoadCursor( hInstance, APP_CURSOR );
#else									// use stock (system) cursor
	wc.hCursor       = LoadCursor( NULL, IDC_ARROW );
#endif
	wc.hbrBackground = GetStockObject( WHITE_BRUSH );
#ifdef APP_MENU
	wc.lpszMenuName  = APP_MENU;		// menu name in .RC/.RES file
#else
	wc.lpszMenuName  = NULL;
#endif
	wc.lpszClassName = szAppName;		// name used in CreateWindow
	return( RegisterClass( &wc ) );		// register class & return result
}

//=======================================================================//
// InitInstance is called -- at initialization time -- for every         //
// instance of the application executed and performs tasks which cannot  //
// be shared across multiple instances.                                  //
//                                                                       //
// In this example, InitInstance saves the instance handle (using a      //
// static variable) before creating and displaying the program's main    //
// window.                                                               //
//=======================================================================//

BOOL InitInstance( HANDLE hInstance,	// current instance identifier
                   int    nCmdShow )	// param for ShowWindow() call
{
	HWND  hWnd;							// main window handle

	hInst = hInstance;		// save current instance handle as static variable
	hWnd = CreateWindow(				// create main window for this instance
				szAppName,				// see RegisterClass() call 
				szAppTitle,				// text for window title bar    
				WS_OVERLAPPEDWINDOW,	// window style             
				CW_USEDEFAULT,			// horizontal position      
				CW_USEDEFAULT,			// vertical position        
				CW_USEDEFAULT,			// horizontal size          
				CW_USEDEFAULT,			// vertical size            
				NULL,					// overlapped = no parent   
				NULL,					// uses window class menu   
				hInstance,				// instance owning window   
				NULL   );				// Pointer not required     
	if( ! hWnd ) return( FALSE );	// if CreateWindow() fails ... 
	ShowWindow( hWnd, nCmdShow );	// make window visible      
	UpdateWindow( hWnd );			// initiate WM_PAINT message    
	return( TRUE );					// return success           
}
