//=======================//
//       PenDraw5.C      //
//  C++ Windows Drawing  //
//=======================//

#include <windows.h>
#include <stdio.h>
#include "pendraw5.h"

#define APP_ICON  "PenDraw5"
#define APP_MENU  "PenDraw5"
// note: no cursor assignment

HANDLE   hInst;
char     szAppTitle[] = "Pen Draw 5: Bitmap Images";
char     szAppName[]  = "PenDraw5";
int      BitmapOp     = IDM_DrawBM;
BOOL     bMoveBitmap  = FALSE;
HBITMAP  hBitmap;
POINT    ptBitmap;

static   int  LineData[] =   {	100, 150, 130, 160, 125, 155,
								125, 140, 110, 120, 135, 140,
								150, 155, 145, 130, 120, 100  };

HCURSOR SwitchCursor( LPSTR lpName )
{
	HCURSOR   hCursor;

	hCursor = LoadCursor( NULL, lpName );
	SetCursor( hCursor );
	return( hCursor );
}

BOOL DrawBitmap( HWND hwnd, LPSTR lpName,
                 int xPos, int yPos )
{
	HDC     hdc, hdcMem;
	BITMAP  bm;
	HBITMAP hBitmap;

	if( !( hBitmap = LoadBitmap( hInst, lpName ) ) )
		return( FALSE );
	hdc = GetDC( hwnd );
	hdcMem = CreateCompatibleDC( hdc );
	SelectObject( hdcMem, hBitmap );
	SetMapMode( hdcMem, GetMapMode( hdc ) );
	GetObject( hBitmap, sizeof(BITMAP), (LPSTR) &bm );
	BitBlt( hdc, xPos, yPos, bm.bmWidth, bm.bmHeight,
			hdcMem, 0, 0, SRCCOPY );
	ReleaseDC( hwnd, hdc );
	DeleteDC( hdcMem );
	DeleteObject( hBitmap );
	return( TRUE );
}

BOOL DrawCenBitmap( HWND hwnd, LPSTR lpName,
                    int xPos, int yPos )
{
	HDC     hdc, hdcMem;
	BITMAP  bm;
	HBITMAP hBitmap;

	if( !( hBitmap = LoadBitmap( hInst, lpName ) ) )
		return( FALSE );
	hdc = GetDC( hwnd );
	hdcMem = CreateCompatibleDC( hdc );
	SelectObject( hdcMem, hBitmap );
	SetMapMode( hdcMem, GetMapMode( hdc ) );
	GetObject( hBitmap, sizeof(BITMAP), (LPSTR) &bm );
	xPos -= bm.bmWidth/2;                                // center bitmap  //
	yPos -= bm.bmHeight/2;                               // on coordinates //
	BitBlt( hdc, xPos, yPos, bm.bmWidth, bm.bmHeight,
			hdcMem, 0, 0, SRCCOPY );
	ReleaseDC( hwnd, hdc );
	DeleteDC( hdcMem );
	DeleteObject( hBitmap );
	return( TRUE );
}

void LineGraph( HWND hwnd, LPSTR lpName )
{
	int i, j;
	
	j = sizeof(LineData) / sizeof(int);
	for( i=0; i<(sizeof(LineData)/sizeof(int)); i++ )
		DrawCenBitmap( hwnd, lpName, i*25+20, LineData[i] );
}

BOOL StretchBitmap( HWND hwnd, LPSTR lpName, RECT rect )
{
	HDC     hdc, hdcMem;
	HBITMAP hBitmap;
	BITMAP  bm;

	if( !( hBitmap = LoadBitmap( hInst, lpName ) ) )
		return( FALSE );
	hdc = GetDC( hwnd );
	hdcMem = CreateCompatibleDC( hdc );
	SelectObject( hdcMem, hBitmap );
	SetMapMode( hdcMem, GetMapMode(hdc) );
	GetObject( hBitmap, sizeof(BITMAP), (LPSTR) &bm );
	StretchBlt( hdc, rect.left,  rect.top, rect.right, rect.bottom,
				hdcMem, 0, 0, bm.bmWidth, bm.bmHeight, SRCCOPY );
	DeleteDC( hdcMem );
	ReleaseDC( hwnd, hdc );
	DeleteObject( hBitmap );
	return( TRUE );
}

BOOL StretchBitMap2Client( HWND hwnd, LPSTR lpName )
{
	HDC     hdc, hdcMem;
	HBITMAP hBitmap;
	BITMAP  bm;
	RECT    rect;

	if( !( hBitmap = LoadBitmap( hInst, lpName ) ) )
		return( FALSE );
	hdc = GetDC( hwnd );
	hdcMem = CreateCompatibleDC( hdc );
	SelectObject( hdcMem, hBitmap );
	SetMapMode( hdcMem, GetMapMode(hdc) );
	GetObject( hBitmap, sizeof(BITMAP), (LPSTR) &bm );
	GetClientRect( hwnd, &rect );
	StretchBlt( hdc, rect.left, rect.top, rect.right, rect.bottom,
				hdcMem, 0, 0, bm.bmWidth, bm.bmHeight, SRCCOPY );
	DeleteDC( hdcMem );
	ReleaseDC( hwnd, hdc );
	DeleteObject( hBitmap );
	return( TRUE );
}

void MoveBitmap( HWND hwnd, HBITMAP hBitmap, POINT pt )
{
	HDC    hdc, hdcMem;
	BITMAP bm;

	hdc = GetDC( hwnd );
	hdcMem = CreateCompatibleDC( hdc );
	SelectObject( hdcMem, hBitmap );
	SetMapMode( hdcMem, GetMapMode(hdc) );
	GetObject( hBitmap, sizeof(BITMAP), (LPSTR) &bm );
	BitBlt( hdc, pt.x, pt.y, bm.bmWidth, bm.bmHeight,
			hdcMem, 0, 0, SRCCOPY );
	ReleaseDC( hwnd, hdc );
	DeleteDC( hdcMem );
}

void ResetCaption( HWND  hwnd )
{
	switch( BitmapOp )
	{
		case IDM_DrawBM:
			SetWindowText( hwnd, "Bitmap Coordinates At UpLeft" );
			break;

		case IDM_CenterBM:
			SetWindowText( hwnd, "Bitmap Centered On Coordinates" );
			break;

		case IDM_StretchBM:
			SetWindowText( hwnd, "Bitmap Stretched To Fit Rectangle" );
			break;

		case IDM_Stretch2Client:
			SetWindowText( hwnd, "Bitmap Stretched To Client Window" );
			break;

		case IDM_BMTrackMouse:
			SetWindowText( hwnd, "Bitmap Tracks Mouse (LButton Down)" );
			break;

		case IDM_BMLineGraph:
			SetWindowText( hwnd, "Simple Line Graph Using Bitmaps" );
			break;
	}
	return;
}

long APIENTRY WndProc( HWND hwnd,   UINT msg,
                       UINT wParam, LONG lParam )

{
	static  int  cxStep, cyStep;
	HDC          hdc;
	HMENU        hMenu;
	RECT         rect;
	PAINTSTRUCT  ps;

	switch( msg )
	{
		case WM_COMMAND:
			hMenu = GetMenu( hwnd );
			switch( LOWORD( wParam ) )
			{
				case IDM_DrawBM:
				case IDM_CenterBM:
				case IDM_StretchBM:
				case IDM_Stretch2Client:
				case IDM_BMTrackMouse:
				case IDM_BMLineGraph:
					CheckMenuItem( hMenu, BitmapOp, MF_UNCHECKED );
					BitmapOp = wParam;
					CheckMenuItem( hMenu, BitmapOp, MF_CHECKED );
					ResetCaption( hwnd );
					InvalidateRect( hwnd, NULL, TRUE );
					break;

				default:
					return( DefWindowProc( hwnd, msg, wParam, lParam ) );
			}
			break;

		case WM_SIZE:
			cxStep = LOWORD( lParam )/4;
			cyStep = HIWORD( lParam )/4;
			InvalidateRect( hwnd, NULL, TRUE );
			break;

		case WM_MOUSEMOVE:								// fall through
			if( !( wParam && MK_LBUTTON ) ) break;      // if button down

		case WM_LBUTTONDOWN:
			if( BitmapOp == IDM_BMTrackMouse )
			{
				InvalidateRect( hwnd, NULL, TRUE );      // erase screen
				ptBitmap.x = LOWORD( lParam );           // update position
				ptBitmap.y = HIWORD( lParam );
			}
			break;

		case WM_PAINT:
			hdc = BeginPaint( hwnd, &ps );
			// ...
			// all other paint operations here //
			// ...
			EndPaint( hwnd, &ps );
			switch( BitmapOp )
			{
				case IDM_DrawBM:
					DrawBitmap( hwnd, "BOX",			cxStep,	  cyStep   );
					DrawBitmap( hwnd, "OIL_DROP",		cxStep,	  cyStep*3 );
					DrawBitmap( hwnd, "ELECTRIC",		cxStep*3, cyStep   );
					DrawBitmap( hwnd, "WHEEL",			cxStep*3, cyStep*3 );
					DrawBitmap( hwnd, "AUTHOR_LOGO",	cxStep*2, cyStep*2 );
					break;

				case IDM_CenterBM:
					DrawCenBitmap( hwnd, "BOX",			cxStep,   cyStep   );
					DrawCenBitmap( hwnd, "OIL_DROP",	cxStep,   cyStep*3 );
					DrawCenBitmap( hwnd, "ELECTRIC",	cxStep*3, cyStep   );
					DrawCenBitmap( hwnd, "WHEEL",		cxStep*3, cyStep*3 );
					DrawCenBitmap( hwnd, "AUTHOR_LOGO",	cxStep*2, cyStep*2 );
					break;

				case IDM_StretchBM:
					SwitchCursor( IDC_WAIT );
					SetRect( &rect, cxStep, cyStep, 2*cxStep, 2*cyStep );
					StretchBitmap( hwnd, "BOX", rect );
					SwitchCursor( IDC_ARROW );
					break;

				case IDM_Stretch2Client:
					SwitchCursor( IDC_WAIT );
					StretchBitMap2Client( hwnd, "AUTHOR_LOGO" );
					SwitchCursor( IDC_ARROW );
					break;

				case IDM_BMTrackMouse:
					if( !bMoveBitmap )
					{
						ptBitmap.x = 10;
						ptBitmap.y = 10;
						bMoveBitmap = TRUE;
						hBitmap = LoadBitmap( hInst, "OIL_DROP" );
					}
					MoveBitmap( hwnd, hBitmap, ptBitmap );
					break;

				case IDM_BMLineGraph:
					LineGraph( hwnd, "ELECTRIC" );
					break;
			}
			break;

		case WM_DESTROY:  PostQuitMessage(0);  break;

		default:	return( DefWindowProc( hwnd, msg, wParam, lParam ) );
	}
	return 0;
}

#include "template.i"

int APIENTRY WinMain( HANDLE hInstance, HANDLE hPrevInstance,
                      LPSTR  lpCmdLine, int    nCmdShow    )
{
	MSG   msg;

	if( ! hPrevInstance )
		if( ! InitApplication( hInstance ) )
			return( FALSE );
	if( ! InitInstance( hInstance, nCmdShow ) )
		return (FALSE);
	while( GetMessage( &msg, NULL, 0, 0 ) )
	{
		TranslateMessage( &msg );
		DispatchMessage( &msg );
	}
	return( msg.wParam );
	UNREFERENCED_PARAMETER( lpCmdLine );
}
