//=====================//
//       Fonts.C       //
//  C++ Windows Fonts  //
//=====================//

#include <windows.h>
#include <string.h>
#include "fonts.h"

#define APP_ICON  "Fonts"               // icon name
#define APP_MENU  "Fonts"               // main menu
// note: no cursor assignment

typedef struct tagFONTLIST
{	GLOBALHANDLE  hGMem;
	int           nCount;  } FONTLIST;

HANDLE  hInst;      // global handle for current instance
HWND    hDlg;       // global handle for dialog box     
char    szAppTitle[] = "Font Enumeration Demo";
char    szAppName[]  = "FONTS";
int     nMapMode     = IDD_TEXT;
DWORD   dwAspMatch   = 0L;
LOGFONT lf = {  0, 0, 0, 0, 0, 0, 0,
                0, 0, 0, 0, 0, 0, "" };

void SetMapModeProc( HDC hdc )
{
	if( nMapMode != IDD_LOGTWIPS )
		SetMapMode( hdc, nMapMode - IDD_TEXT + MM_TEXT );
	else
	{
		SetMapMode( hdc, MM_ANISOTROPIC );
		SetWindowExtEx( hdc, 1440, 1440, NULL );
		SetViewportExtEx( hdc,
			GetDeviceCaps( hdc, LOGPIXELSX ),
			GetDeviceCaps( hdc, LOGPIXELSY ), NULL );
	}
}

void ShowMetrics( HWND hDlg )
{
	char   szFontName[LF_FACESIZE];
	BOOL   bTrans;
	HFONT  hFont;
	HDC    hdc;
	int    i; 
	static TEXTMETRIC tm;
	static struct
	{
		int  nDlgID;
		int *pTM;   
	}	FontData[] =
			{  
				IDD_TM_HEIGHT,   &tm.tmHeight,
				IDD_TM_ASCENT,   &tm.tmAscent,
				IDD_TM_DESCENT,  &tm.tmDescent,
				IDD_TM_LEADINT,  &tm.tmInternalLeading,
				IDD_TM_LEADEXT,  &tm.tmExternalLeading,
				IDD_TM_WIDTHAVG, &tm.tmAveCharWidth,
				IDD_TM_WIDTHMAX, &tm.tmMaxCharWidth,
				IDD_TM_WEIGHT,   &tm.tmWeight,
				IDD_TM_OVER,     &tm.tmOverhang,
				IDD_TM_DIGITX,   &tm.tmDigitizedAspectX,
				IDD_TM_DIGITY,   &tm.tmDigitizedAspectY   };

	lf.lfHeight = GetDlgItemInt( hDlg, IDD_HEIGHT, &bTrans, TRUE  );
	lf.lfWidth  = GetDlgItemInt( hDlg, IDD_WIDTH,  &bTrans, FALSE );
	lf.lfWeight = GetDlgItemInt( hDlg, IDD_WEIGHT, &bTrans, FALSE );
	GetDlgItemText( hDlg, IDD_TM_NAME, lf.lfFaceName, LF_FACESIZE );
	hdc = GetDC( hDlg );
	SetMapModeProc( hdc );
	SetMapperFlags( hdc, dwAspMatch );
	hFont = SelectObject( hdc, CreateFontIndirect( &lf ) );
	GetTextMetrics( hdc, &tm );
	SetDlgItemInt( hDlg, IDD_HEIGHT, tm.tmHeight, FALSE );
	SetDlgItemInt( hDlg, IDD_WIDTH,  tm.tmAveCharWidth, FALSE );
	SetDlgItemInt( hDlg, IDD_WEIGHT, tm.tmWeight, FALSE );
	GetTextFace( hdc, sizeof( szFontName ), szFontName );
	SendDlgItemMessage( hDlg, IDD_TM_NAME, WM_SETTEXT,
						0, (DWORD)(LPSTR) szFontName );
	DeleteObject( SelectObject( hdc, hFont ) );
	ReleaseDC( hDlg, hdc );
	for( i=0; i<sizeof(FontData)/sizeof(FontData[0]); i++ )
		SetDlgItemInt( hDlg, FontData[i].nDlgID, *FontData[i].pTM, TRUE );
	return;
}

int APIENTRY FontEnumFunc( LPLOGFONT      lf,
                           LPTEXTMETRIC   tm,
                           short          nFontType,
                           FONTLIST FAR * FontList )
{
	LPSTR lpFontFace;

	if( ! GlobalReAlloc( FontList->hGMem,
				 (DWORD) LF_FACESIZE * ( FontList->nCount + 1 ),
						 GMEM_MOVEABLE ) )
		return( FALSE );
	lpFontFace = GlobalLock( FontList->hGMem );
	lstrcpy( lpFontFace + ( ( FontList->nCount ) * LF_FACESIZE ),
			 (LPSTR) lf->lfFaceName );
	GlobalUnlock( FontList->hGMem );
	FontList->nCount++;
	return( TRUE );
} 

BOOL APIENTRY DlgProc( HWND hDlg, UINT msg, UINT wParam, LONG lParam )
{
	static FARPROC  lpEnumProc;
	static FONTLIST FontList;
	LPSTR           lpFontName;
	HDC             hdc; 
	int             i, nSel;
	char            szFont[LF_FACESIZE];

	switch( msg )
	{
		case WM_INITDIALOG:
			CheckRadioButton( hDlg, IDD_TEXT, IDD_LOGTWIPS, IDD_TEXT );
			lf.lfEscapement    = 0;
			lf.lfOrientation   = 0;
			lf.lfOutPrecision  = OUT_DEFAULT_PRECIS;
			lf.lfClipPrecision = CLIP_DEFAULT_PRECIS;

			FontList.hGMem = GlobalAlloc( GMEM_MOVEABLE | GMEM_ZEROINIT, 1L );
			FontList.nCount = 0;

			lpEnumProc = MakeProcInstance( FontEnumFunc, hInst );
			hdc = GetDC( GetParent( hDlg ) );
			EnumFonts( hdc, NULL, lpEnumProc, (LPARAM) &FontList );
			lpFontName = GlobalLock( FontList.hGMem );
			SendDlgItemMessage( hDlg, IDD_FONTLIST, WM_SETREDRAW,
			                    (WPARAM) 1, (LPARAM) 0 );
			SendDlgItemMessage( hDlg, IDD_FONTLIST, LB_RESETCONTENT, 0, 0 );
			for( i=0; i<FontList.nCount; i++ )
				SendDlgItemMessage( hDlg, IDD_FONTLIST, LB_ADDSTRING, 0,
					(LPARAM)(LPSTR)( lpFontName + ( i * LF_FACESIZE ) ) );
			GlobalUnlock( FontList.hGMem );
			GlobalFree( FontList.hGMem );
			FreeProcInstance( lpEnumProc );
			ReleaseDC( GetParent( hDlg ), hdc );
			ShowMetrics( hDlg );  // fall through to SetFocus

		case WM_SETFOCUS:
			SetFocus( GetDlgItem( hDlg, IDD_HEIGHT ) );
			break;

		case WM_COMMAND:
			switch( LOWORD( wParam ) )
			{
				case IDD_TEXT:
				case IDD_LOMETRIC:
				case IDD_HIMETRIC:
				case IDD_LOENGLISH:
				case IDD_HIENGLISH:
				case IDD_TWIPS:
				case IDD_LOGTWIPS:
					nMapMode = LOWORD( wParam );
					// fall through to IDD_SETFONT

				case IDD_SETFONT:
					nSel = (DWORD) SendDlgItemMessage( hDlg, IDD_FONTLIST,
					                                   LB_GETCURSEL, 0, 0 );
					SendDlgItemMessage( hDlg, IDD_FONTLIST, LB_GETTEXT, nSel,
										(DWORD)(LPSTR) szFont );
					SendDlgItemMessage( hDlg, IDD_TM_NAME, WM_SETTEXT, 0,
										(DWORD)(LPSTR) szFont );
					ShowMetrics( hDlg );
					InvalidateRect( GetParent(hDlg), NULL, TRUE );
					break;

				case IDD_QUIT:
					PostQuitMessage(0);
					break;

			}  break;

		default:  return( FALSE );
	}
	return( TRUE );
}

long APIENTRY WndProc( HWND hwnd,   UINT msg,
                       UINT wParam, LONG lParam )
{
	static FARPROC  lpfnDlgProc;
	static char     szText[] = "The quick brown fox jumps over the lazy red dog. "
							   "1234567890";
	HDC         hdc;
	HFONT       hFont;
	PAINTSTRUCT ps;
	RECT        rect;

	switch( msg )
	{
		case WM_CREATE:
			lpfnDlgProc = MakeProcInstance( DlgProc, hInst );
			hDlg = CreateDialog( hInst, szAppName, hwnd, lpfnDlgProc );
			break;

		case WM_SETFOCUS:
			SetFocus( hDlg );
			break;

		case WM_PAINT:
			hdc = BeginPaint( hwnd, &ps );
			SetMapModeProc( hdc );
			SetMapperFlags( hdc, dwAspMatch );
			GetClientRect( hDlg, &rect );
			rect.bottom += 10;
			DPtoLP( hdc, (LPPOINT) &rect, 2 );
			hFont = SelectObject( hdc, CreateFontIndirect( &lf ) );
			TextOut( hdc, rect.left, rect.bottom, szText, strlen( szText ) );             
			DeleteObject( SelectObject( hdc, hFont ) );
			EndPaint( hwnd, &ps );
			break;

		case WM_DESTROY:
			FreeProcInstance( lpfnDlgProc );
			PostQuitMessage(0);
			break;

		default: return( DefWindowProc( hwnd, msg, wParam, lParam ) );
	}
	return( FALSE );
}

#include "template.i"

int APIENTRY WinMain( HANDLE hInstance, HANDLE hPrevInstance,
                      LPSTR  lpCmdLine, int    nCmdShow    )
{
	MSG   msg;

	if( ! hPrevInstance )
		if( ! InitApplication( hInstance ) )
			return( FALSE );
	if( ! InitInstance( hInstance, nCmdShow ) )
		return (FALSE);
	while( GetMessage( &msg, NULL, 0, 0 ) )
	{
		TranslateMessage( &msg );
		DispatchMessage( &msg ); 
	}
	return( msg.wParam );
	UNREFERENCED_PARAMETER( lpCmdLine );
}
