//==========================//
//        CAPTURE.C         //
//  Screen Capture Program  //
//==========================//

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "CAPTURE.h"

#define APP_ICON  "CAPTURE"
#define APP_MENU  "CAPTURE"
// note: no cursor assignment

HANDLE  hInst;
char    szAppTitle[] = "Screen Capture Demo";
char    szAppName[]  = "CAPTURE";
char    szFName[64]  = "";
int     iSec, Clock;

BOOL ErrorMsg( char * Error )
{
	HWND hwnd = GetFocus();

	MessageBeep( MB_ICONASTERISK );
	MessageBox( hwnd, Error, "Warning", MB_ICONASTERISK | MB_OK );
	return 0;
}

BOOL APIENTRY FileNameDlgProc( HWND hDlg,   UINT msg,
                               UINT wParam, LONG lParam )
{
	switch( msg )
	{
		case WM_INITDIALOG:
			SendDlgItemMessage( hDlg, IDD_FNAME, EM_LIMITTEXT, 80, 0 );
			SetDlgItemText( hDlg, IDD_FNAME, szFName );
			return( TRUE );

		case WM_COMMAND:
			switch( LOWORD( wParam ) )
			{
				case IDD_FNAME:
					if( HIWORD( wParam ) == EN_CHANGE )
						EnableWindow( GetDlgItem( hDlg, IDOK ),
									  (BOOL) SendMessage( (HWND) lParam,
									  WM_GETTEXTLENGTH, 0, 0L ) );
					return( TRUE );

				case IDOK:
					GetDlgItemText( hDlg, IDD_FNAME, szFName, 80 );
					EndDialog( hDlg, TRUE );
					return( TRUE );

				case IDCANCEL:
					EndDialog( hDlg, FALSE );
					return( TRUE );

				default: return( FALSE );
   			}
	}
	return( FALSE );
}

int CaptureBitmap( HWND hwnd )
{
	HDC			hdc, hdcMem;
	HBITMAP		hBitmap;
	static int	i, j, CRes, LnWidth, LnPad = 0,
				xSize, ySize; 

	SetCursor( LoadCursor( NULL, IDC_WAIT ) );
	hdc = GetDC( HWND_DESKTOP );
	xSize = GetDeviceCaps( hdc, HORZRES );
	ySize = GetDeviceCaps( hdc, VERTRES );
	hdcMem = CreateCompatibleDC( hdc );
	hBitmap = CreateCompatibleBitmap( hdc, xSize, ySize );
	if( hBitmap )
	{
		SelectObject( hdcMem, hBitmap );
		StretchBlt( hdcMem, 0, 0, xSize, ySize,
					hdc,    0, 0, xSize, ySize, SRCCOPY );
		OpenClipboard( hwnd );
		EmptyClipboard();
		SetClipboardData( CF_BITMAP, hBitmap );
		CloseClipboard();
		InvalidateRect( hwnd, NULL, TRUE );
	}
	DeleteDC( hdcMem );
	ReleaseDC( HWND_DESKTOP, hdc );
	return 0;
}

int SaveBitmap()
{
	HDC      hdc, hdcMem;
	HANDLE   hBits, hFil;
	HBITMAP  hBitmap;
	HPALETTE hPal;
	LPVOID   lpBits;
	RGBQUAD  RGBQuad;
	DWORD    ImgSize, plSize, dwWritten;
	int      i, CRes, Height, Width, LnWidth, LnPad;
	BITMAPFILEHEADER  bmFH;
	BITMAPINFO        bmInfo;
	LPLOGPALETTE      lp;

	SetCursor( LoadCursor( NULL, IDC_WAIT ) );
		//=== open file for write ================================
	hFil = CreateFile( szFName, GENERIC_WRITE, 0,  NULL,
						CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL );
	if( hFil == NULL )
		return( ErrorMsg( "Can't open file" ) );

	hdc = GetDC( HWND_DESKTOP );
	CRes = GetDeviceCaps( hdc, SIZEPALETTE );
	plSize = CRes * sizeof( RGBQUAD );      // palette size
	bmInfo.bmiHeader.biBitCount = GetDeviceCaps( hdc, BITSPIXEL ); 
	Height = GetDeviceCaps( hdc, VERTRES );
	Width  = GetDeviceCaps( hdc, HORZRES );
	if( GetDeviceCaps( hdc, BITSPIXEL ) == 8 ) 
		LnWidth = Width;
	else
		LnWidth = Width / 2;
	if( LnWidth % sizeof(DWORD) )
		LnPad = sizeof(DWORD) - ( LnWidth % sizeof(DWORD) );
	ImgSize = (DWORD)( (DWORD)( LnWidth + LnPad ) * 480 );

		//=== initialize bitmap file header structure ============
	bmFH.bfType      = 0x4D42;             // Type is "BF"
	bmFH.bfReserved1 = 0L;
	bmFH.bfReserved2 = 0L;
	bmFH.bfOffBits   = plSize +            // bitmap offset
					   sizeof( BITMAPINFO ) +
					   sizeof( BITMAPFILEHEADER );
	bmFH.bfSize      = ImgSize +           // file size
					   bmFH.bfOffBits;
	WriteFile( hFil, &bmFH, sizeof( bmFH ), &dwWritten, NULL );	// write file header

		//=== initialize bitmap info header structure ============
	bmInfo.bmiHeader.biSize = (DWORD) sizeof( BITMAPINFOHEADER );
	bmInfo.bmiHeader.biWidth         = Width;
	bmInfo.bmiHeader.biHeight        = Height;
	bmInfo.bmiHeader.biPlanes        = 1;
	bmInfo.bmiHeader.biCompression   = BI_RGB;
	bmInfo.bmiHeader.biSizeImage     = 0L;
	bmInfo.bmiHeader.biXPelsPerMeter = 0L;
	bmInfo.bmiHeader.biYPelsPerMeter = 0L;
	bmInfo.bmiHeader.biClrUsed       = 0L;
	bmInfo.bmiHeader.biClrImportant  = 0L;
	WriteFile( hFil, &bmInfo.bmiHeader, sizeof( bmInfo.bmiHeader ),
			   &dwWritten, NULL );     // write info header

		//=== add the palette color information ==================
			// note: GHND = GMEM_FIXED | GMEM_ZEROINIT
	hPal = GlobalAlloc( GHND, sizeof(LOGPALETTE) +
                       ( CRes * sizeof(PALETTEENTRY) ) );
			// allocate memory for palette
	lp = (LPLOGPALETTE) GlobalLock( hPal );
			// lock the memory allocated
	lp->palNumEntries = CRes;
	lp->palVersion    = 0x0300;
			// fill in size and version (3.0)
	GetSystemPaletteEntries( hdc, 0, CRes, lp->palPalEntry );
			// and get the palette information
		//========= Structure definitions =====================
	//	Palette Entry			RGB Quad for Bitmap
	//
	//	typedef struct			typedef struct tagRGBQUAD 
	//	{						{
	//		BYTE  peRed;			BYTE  rgbBlue;
	//		BYTE  peGreen;			BYTE  rgbGreen;
	//		BYTE  peBlue;			BYTE  rgbRed;
	//		BYTE  peFlags;			BYTE  rgbReserved;
	//	} PALETTEENTRY;			} RGBQUAD;
	//
	//=====================================================
         // record each PALETTEENTRY as RGBQUAD format
	RGBQuad.rgbReserved = 0;
	for( i=0; i<=CRes; i++ )
	{
		RGBQuad.rgbRed   = lp->palPalEntry[i].peRed;
		RGBQuad.rgbGreen = lp->palPalEntry[i].peGreen;
		RGBQuad.rgbBlue  = lp->palPalEntry[i].peBlue;
		WriteFile( hFil, &RGBQuad, sizeof( RGBQuad ), &dwWritten, NULL );
	}
	GlobalUnlock( hPal );     // don't forget to unlock
	GlobalFree( hPal );       // and release the memory
		//====== write image data to file ========================
	hdcMem = CreateCompatibleDC( hdc );
	hBitmap = CreateCompatibleBitmap( hdc, Width, 1 );
	hBits = GlobalAlloc( GHND, LnWidth );
	lpBits = (LPVOID) GlobalLock( hBits );
	SelectObject( hdcMem, hBitmap );
	for( i = Height - 1; i >= 0; i-- )
	{
		BitBlt( hdcMem, 0, 0, Width, 1, hdc, 0, i, SRCCOPY );
		GetBitmapBits( hBitmap, Width, lpBits );
		WriteFile( hFil, lpBits, LnWidth, &dwWritten, NULL );
	}
	GlobalUnlock( hBits );			// don't forget to unlock
	GlobalFree( hBits );			// and release the image,
	DeleteDC( hdcMem );				// delete and release the
	ReleaseDC( HWND_DESKTOP, hdc );	// device contexts
	CloseHandle( hFil );			// and close the file
	SetCursor( LoadCursor( NULL, IDC_ARROW ) );
	return( TRUE );
}

long APIENTRY WndProc( HWND hwnd,   UINT msg,
                       UINT wParam, LONG lParam )
{
	static  BOOL  fExpand = FALSE;
	static  int   cxWnd, cyWnd, iStrMode, Action;
	BITMAP        bm;
	HBITMAP       hBitmap;
	HDC           hdc, hdcMem;
	PAINTSTRUCT   ps;

	switch( msg )
	{
		case WM_SIZE:
			cxWnd = LOWORD( lParam );
			cyWnd = HIWORD( lParam );
			InvalidateRect( hwnd, NULL, TRUE );
			break;

		case WM_TIMER:
			if( ++iSec == 7 )
				PostMessage( hwnd, WM_COMMAND, IDM_CAPTURE, 0 );
			else 
				MessageBeep( MB_ICONEXCLAMATION );
			break;

		case WM_COMMAND:
			switch( LOWORD( wParam ) )
			{
				case IDM_EXACT:
					fExpand = FALSE;
					InvalidateRect( hwnd, NULL, TRUE );
					break;

				case IDM_RESIZE:
					fExpand = TRUE;
					InvalidateRect( hwnd, NULL, TRUE );
					break;

				case IDM_CLIP:
					Action = TOCLIPBD;
					CloseWindow( hwnd );
					Clock = SetTimer( hwnd, 1, 1000, NULL );
					iSec = 0;
					break;

				case IDM_FILE:
					if( DialogBox( hInst, "GETNAME", hwnd, FileNameDlgProc ) )
					{
						Action = TOFILE;
						CloseWindow( hwnd );
						Clock = SetTimer( hwnd, 1, 1000, NULL );
						iSec = 0;
					}
					break;

				case IDM_CAPTURE:
					MessageBeep( MB_OK );
					KillTimer( hwnd, Clock );
					SetCursor( LoadCursor( NULL, IDC_WAIT ) );
					switch( Action )
					{
						case TOFILE:	SaveBitmap();          break;
						case TOCLIPBD:	CaptureBitmap( hwnd ); break;
					}
					SetCursor( LoadCursor( NULL, IDC_ARROW ) );
					OpenIcon( hwnd );
					break;

				default:
					return( DefWindowProc( hwnd, msg, wParam, lParam ) );
			}
			break;

		case WM_PAINT:
			hdc = BeginPaint( hwnd, &ps );
			OpenClipboard( hwnd );
			if( hBitmap = GetClipboardData( CF_BITMAP ) )
			{
				SetCursor( LoadCursor( NULL, IDC_WAIT ) );
				hdcMem = CreateCompatibleDC( hdc );
				SelectObject( hdcMem, hBitmap );
				GetObject( hBitmap, sizeof( BITMAP ), (LPSTR) &bm );
				if( fExpand )
				{
					SetStretchBltMode( hdc, iStrMode );
					StretchBlt( hdc, 0, 0, cxWnd, cyWnd,
								hdcMem, 0, 0, bm.bmWidth, bm.bmHeight, SRCCOPY );
				}
				else
					BitBlt( hdc, 0, 0, cxWnd, cyWnd, hdcMem, 0, 0, SRCCOPY );
				SetCursor( LoadCursor( NULL, IDC_ARROW ) );
				DeleteDC( hdcMem );
			}
			CloseClipboard();
			EndPaint( hwnd, &ps );
			break;

		case WM_DESTROY:
			PostQuitMessage(0);
			break;

		default: return( DefWindowProc( hwnd, msg, wParam, lParam ) );
	}
	return 0;
}

#include "template.i"

int APIENTRY WinMain( HANDLE hInstance, HANDLE hPrevInstance,
                      LPSTR  lpCmdLine, int    nCmdShow    )
{
	MSG   msg;

	if( ! hPrevInstance )
		if( ! InitApplication( hInstance ) )
			return( FALSE );
	if( ! InitInstance( hInstance, nCmdShow ) )
		return (FALSE);
	while( GetMessage( &msg, NULL, 0, 0 ) )
	{
		TranslateMessage( &msg );
		DispatchMessage( &msg );
	}
	return( msg.wParam );
	UNREFERENCED_PARAMETER( lpCmdLine );
}