//========================//
//       VIEWPCX.C        //
//  256-Color PCX Viewer  //
//========================//

#include <windows.h>
#include <string.h>
#include "viewpcx.h"
#include "pcxhead.i"			// PCX image header definition

#define wFileAttr  0x4010		// file attribute flag

#define APP_ICON  "ViewPCX"		// icon name
#define APP_MENU  "ViewPCX"		// main menu
	// note: no cursor assignment
                                                             
HANDLE	hInst;					// global handle for current instance    
char	szAppTitle[] = "256-Color PCX Viewer";
char	szAppName[]  = "ViewPCX";		// window class name

HANDLE	hInst;					// application hInstance
HWND	hwnd;					// hwnd of main window
char	szFileSpec[128],		// initial file spec
		szFileExt[5] = ".PCX",	// default extension
		szFileName[128] = "*",	// file name
		szFilePath[128],		// file path
		szFName[128] = "";		// file to display
WORD	wStatus;				// status of search
BOOL	bHaveImage;

int ErrorMsg( char *Message )
{
	MessageBox( GetFocus(), Message, "Error!", MB_ICONQUESTION | MB_OK );
	return(0);
}

//======================================================//
// FileOpen - get the name of a file to open            //
//======================================================//

BOOL APIENTRY FileOpen( HWND hDlg,   UINT msg,
                        UINT wParam, LONG lParam )
{
	static char   OrgPath[128];
	char          cLastChar;
	int           nLen;

	switch (msg)
	{
		case WM_INITDIALOG:
			GetCurrentDirectory( sizeof(OrgPath), OrgPath );  
			SendDlgItemMessage( hDlg, IDD_FNAME, EM_LIMITTEXT, 80, 0L );
				// fill list box using starting file spec
			DlgDirList( hDlg, szFileSpec, IDD_FLIST, IDD_FPATH, wFileAttr );
				// show init filespec
			SetDlgItemText( hDlg, IDD_FNAME, szFileSpec );
			return( TRUE );

		case WM_COMMAND:
			switch( LOWORD( wParam ) )
			{
				case IDD_FLIST:
					switch( HIWORD( wParam ) )
					{
						case LBN_SELCHANGE:
							if( DlgDirSelectEx( hDlg, szFileName, 
												sizeof(szFileName), IDD_FLIST ) )
								lstrcat( szFileName, szFileSpec );
						SetDlgItemText( hDlg, IDD_FNAME, szFileName );
						return( TRUE );

						case LBN_DBLCLK:
							if( DlgDirSelectEx( hDlg, szFileName,
												sizeof(szFileName), IDD_FLIST ) )
							{
								lstrcat( szFileName, szFileSpec );
								DlgDirList( hDlg, szFileName, IDD_FLIST, IDD_FPATH,
											wFileAttr );
								SetDlgItemText( hDlg, IDD_FNAME, szFileSpec );
							}
							else
							{
								SetDlgItemText( hDlg, IDD_FNAME, szFileName );
								SendMessage( hDlg, WM_COMMAND, IDOK, 0L );
							}
							return( TRUE );
					}  
					break;

				case IDD_FNAME:
					if( HIWORD( wParam ) == EN_CHANGE )
					{
						EnableWindow( GetDlgItem( hDlg, IDOK ),
							   (BOOL) SendMessage( (HWND) lParam,
												   WM_GETTEXTLENGTH, 0, 0L ));
						return( TRUE );
					}
					else
						return( FALSE );

				case IDOK:
					GetDlgItemText( hDlg, IDD_FNAME, szFileName, 128 );
					nLen  = lstrlen( szFileName );
					cLastChar = *AnsiPrev( szFileName, szFileName + nLen );
					if( ( cLastChar != '\\' ) && ( cLastChar != ':' ) )
						lstrcat( szFileName, "\\" );
					else 
						lstrcat( szFileName, szFileSpec );
					if( strchr( szFileName, '*' ) || strchr( szFileName, '?' ) )
					{
						if( DlgDirList( hDlg, szFileName, IDD_FLIST, IDD_FPATH,
										wFileAttr ) )
						{
							lstrcpy( szFileSpec, szFileName );
							SetDlgItemText( hDlg, IDD_FNAME, szFileSpec );
						}
						else 
							MessageBeep(0);
						return( TRUE );
					}
					if( DlgDirList( hDlg, szFileName, IDD_FLIST, IDD_FPATH,
									wFileAttr ) )
					{
						lstrcpy( szFileSpec, szFileName );
						SetDlgItemText( hDlg, IDD_FNAME, szFileSpec );
						return( TRUE );
					}
					szFileName[nLen] = '\0';
					GetDlgItemText( hDlg, IDD_FPATH, szFilePath, 128 );
					CharUpperBuff( szFilePath, lstrlen( szFilePath ) );
					CharUpperBuff( szFileName, lstrlen( szFileName ) );
					SetCurrentDirectory( OrgPath );          // reset directory
					EndDialog( hDlg, TRUE );                 // end dialog box
					return( TRUE );

				case IDCANCEL:
					SetCurrentDirectory( OrgPath );          // reset directory
					EndDialog( hDlg, FALSE );                // end dialog box
					return( TRUE );

			}  
			break;
	}
	return 0;
}

//======================================================//
// CallFileOpenDlg: invokes FileOpen dialog to get      //
//                  name of file to open                //
// parameters: szFileSpecIn  - initial file path        //
//             szFileExtIn   - initial file extension   //
// (returned)  szFilePathOut - selected file path       //
// (returned)  szFileNameOut - selected file name       //
//     return: TRUE if file selected, FALSE if not      //
//======================================================//

int CallFileOpenDlg( HANDLE hInst, HWND  hwnd,
                     char *pszFileSpecIn,  char *pszFileExtIn,
                     char *pszFilePathOut, char *pszFileNameOut )
{
	int       iReturn;

	strcpy( szFileSpec, pszFileSpecIn);   // save file spec
	strcpy( szFileExt, pszFileExtIn);
	iReturn = DialogBox( hInst, "OPENFILE", hwnd, FileOpen );
	strcpy( pszFilePathOut, szFilePath ); // return filepath
	strcpy( pszFileNameOut, szFileName ); // return filename
	return( iReturn );
}

BOOL PaintImage( HWND hwnd, HDC hdc, char *PCXFile )
{
	short			i, j, k, width, depth, count;
	BYTE			ImgArray[640], Index;
	PCXHEAD			pcxHd;
	HANDLE			hFile;
	DWORD			fSize, fRes;
	OVERLAPPED		fPos;
	LPLOGPALETTE	lPal;
	HPALETTE		hPCXPal, hOldPal;
	char			chPal, pcxPal[768], szBuff[50];
	BOOL			bResult;	
	LPVOID			lpMsgBuf;

	if( strlen( PCXFile ) == 0 ) return(0);
	hFile = CreateFile( PCXFile, GENERIC_READ, 0, NULL,
						OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL ); 
	if( hFile == INVALID_HANDLE_VALUE )
	{
		wsprintf( szBuff, "Error: %d -- unable to open %s!",
				  GetLastError(), PCXFile );
		ErrorMsg( szBuff );
		bHaveImage = FALSE;
		return( FALSE );
	}
	bResult = ReadFile( hFile, &pcxHd, sizeof(PCXHEAD), &fRes, NULL );
	if( ( fRes != sizeof(PCXHEAD) ) || ( pcxHd.manufacturer != 0x0A ) )
	{
		CloseHandle( hFile );
		ErrorMsg( "Not a valid .PCX file" );
		bHaveImage = FALSE;
		return( FALSE );
	}
	width = ( pcxHd.xmax - pcxHd.xmin ) + 1;   // optional
	depth = ( pcxHd.ymax - pcxHd.ymin ) + 1;   // necessary
	fSize = GetFileSize( hFile, NULL );
	if( fSize < ( 769 + sizeof(PCXHEAD) ) )
	{     // wrong format -- too small for palette
		CloseHandle( hFile );
		ErrorMsg( "Not a 256 color image format" );
		bHaveImage = FALSE;
		return( FALSE );
	}
	fPos.Offset = fSize - 769;     // seek palette start
	fPos.OffsetHigh = 0L;
	fPos.hEvent = NULL;
	bResult = ReadFile( hFile, &chPal, 1, &fRes, &fPos );	// get palette ID 
	if( ! bResult )
	{
		CloseHandle( hFile );
		FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
					   NULL, GetLastError(), MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
					   (LPTSTR) &lpMsgBuf, 0, NULL );
		ErrorMsg( lpMsgBuf );
		LocalFree( lpMsgBuf );
		bHaveImage = FALSE;
		return( FALSE );
	}
	fPos.Offset++;                 // advance pointer
	if( ! ReadFile( hFile, &pcxPal, 768, &fRes, &fPos ) )		// get palette
	{
		CloseHandle( hFile );
		wsprintf( szBuff, "Error: %d -- file read error %s!",
				  GetLastError(), PCXFile );
		ErrorMsg( szBuff );
		bHaveImage = FALSE;
		return( FALSE );
	}
	if( ( pcxHd.version != 5 ) ||  // check version number
		( chPal != 0x000C ) ||     // check palette ID
		( fRes != 768 ) )          // check palette size
	{
		CloseHandle( hFile );
		ErrorMsg( "Not a 256 color image format" );
		bHaveImage = FALSE;
		return( FALSE );
	}
	SetCursor( LoadCursor( NULL, IDC_WAIT ) );
		//========= create palette ============================
	hPCXPal = GlobalAlloc( GHND, sizeof(LOGPALETTE) + 
						   256 * sizeof(PALETTEENTRY ) );
	lPal = (LPLOGPALETTE) GlobalLock( hPCXPal );
	lPal->palVersion = 0x0300;
	lPal->palNumEntries = 256;
	for( i=j=0; i<256; ++i )
	{
		lPal->palPalEntry[i].peRed   = pcxPal[j++];
		lPal->palPalEntry[i].peGreen = pcxPal[j++];
		lPal->palPalEntry[i].peBlue  = pcxPal[j++];
		lPal->palPalEntry[i].peFlags = PC_NOCOLLAPSE;
			//  use PC_NOCOLLAPSE instead of PC_RESERVED -- //
			//  PC_RESERVED maps to nearest existing color  //
			//  but no good matches exist for this purpose  //
	}
	hPCXPal = CreatePalette( lPal );
	hOldPal = SelectPalette( hdc, hPCXPal, FALSE );
	RealizePalette( hdc );         // palette is now active
		//========= reset file pointer to image ===============
	fPos.Offset = 128;      // set file ptr to image data
	j = Index = 0;			// initialize position
	while( j < depth )
	{
		fPos.Offset += Index;
		i = Index = 0;
		ReadFile( hFile, &ImgArray, sizeof(ImgArray), &fRes, &fPos );
		do
		{
			if( ( ImgArray[Index] & 0xC0 ) == 0xC0 )
			{
				count = ImgArray[Index++] & 0x3F;
				for( k=0; k<count; k++ )
				{
					SetPixelV( hdc, i++, j, PALETTEINDEX( ImgArray[Index] ) );
					if( i >= pcxHd.bytes_per_line ) k = count;
				}   // if line's too long just ignore any wraps
			}
			else
				SetPixelV( hdc, i++, j, PALETTEINDEX( ImgArray[Index] ) );
			Index++;
		}
		while( i < pcxHd.bytes_per_line );
		j++;
		i = 0;
	} 
	SetCursor( LoadCursor( NULL, IDC_ARROW ) );
		//========= restore original palette ==================
	SelectPalette( hdc, hOldPal, FALSE );
	GlobalUnlock( hPCXPal );
	GlobalFree( hPCXPal );
	CloseHandle( hFile );
	return( TRUE );
}

#ifdef __BORLANDC__
#pragma argsused
#endif

BOOL APIENTRY AboutProc( HWND hDlg,   UINT msg,
                         UINT wParam, LONG lParam )
{
	switch( msg )
	{
		case WM_INITDIALOG: return( TRUE );
		case WM_COMMAND:
			switch( LOWORD( wParam ) )
			{
				case IDOK: EndDialog( hDlg, TRUE ); return( TRUE );
				default:							return( FALSE );
			}     
	}
	return( FALSE );
}

#ifdef __BORLANDC__
#pragma argsused
#endif

long APIENTRY WndProc( HWND hwnd,   UINT msg,
                       UINT wParam, LONG lParam )
{
	static char szTmpFilePath[64], szTmpFileExt[5],
				szTmpFileSpec[64], szTmpFileName[64];
	HDC         hdc;
	PAINTSTRUCT ps;

	switch( msg )
	{
		case WM_CREATE:
			GetCurrentDirectory( sizeof(szTmpFilePath), szTmpFilePath );
			strcat( szTmpFilePath, "\\" );
			strcpy( szTmpFileExt,  szFileExt );
			strcpy( szTmpFileName, "" );
			bHaveImage = FALSE;
			break;

		case WM_COMMAND:
			switch( LOWORD( wParam ) )
			{
				case IDM_ABOUT:
					DialogBox( hInst, "ABOUT", hwnd, AboutProc );
					break;

				case IDM_OPEN:    // setup initial search path
					wsprintf( szTmpFileSpec, "%s\\*%s", szTmpFilePath, szFileExt );
					if( CallFileOpenDlg( hInst, hwnd, szTmpFileSpec, szFileExt,
										 szTmpFilePath, szTmpFileName ) )
					{
						bHaveImage = TRUE;
						wsprintf( szFName, "%s\\%s", szTmpFilePath, szTmpFileName );
						SetWindowText( hwnd, szFName );
						InvalidateRect( hwnd, NULL, TRUE );
						UpdateWindow( hwnd );
					}
					else
						bHaveImage = FALSE;
					break;

				case IDM_QUIT:
					PostMessage( hwnd, WM_CLOSE, 0, 0L );
					break;

				default:
					return( DefWindowProc( hwnd, msg, wParam, lParam ) );

			}  
			break;

		case WM_PAINT:
			InvalidateRect( hwnd, NULL, TRUE );
			hdc = BeginPaint( hwnd, &ps );
			if( bHaveImage )
				PaintImage( hwnd, hdc, szFName );
			EndPaint( hwnd, &ps );
			break;

		case WM_DESTROY:
			PostQuitMessage(0);
			break;

		default: return( DefWindowProc( hwnd, msg, wParam, lParam ) );
	}
	return 0;
}

#include "template.i"

int PASCAL WinMain( HANDLE hInstance,	 HANDLE hPrevInstance,
                    LPSTR  lpszCmdParam, int    nCmdShow )
{
	MSG  msg;

	if( ! hPrevInstance )					// are any other instances running?
		if( ! InitApplication( hInstance ) ) 
			return( FALSE );
	strcpy( szFileExt, ".PCX" );			// set default extension
	strcpy( szFileSpec, "*" );				// set default filespec
	if( ! InitInstance( hInstance, nCmdShow ) ) 
		return (FALSE);
	ShowWindow(   hwnd, nCmdShow );
	UpdateWindow( hwnd );
	while( GetMessage( &msg, NULL, 0, 0 ) )
	{
		TranslateMessage( &msg );			// translate virtual key codes
		DispatchMessage( &msg );			// dispatch message to window 
	}
	return( msg.wParam );
	UNREFERENCED_PARAMETER( lpszCmdParam ); // prevents unused para warning
}											//    message from compiler
