//====================================//
//             DIBitMap.I             //
//  routines to read DIB bitmap file  //
//====================================//

//=== global variables ===============//

DWORD         bmImgSize,  bmScanWidth, bmWidth, bmHeight;
HBITMAP       hGBM  = 0;
HPALETTE      hGPal = 0;
LPLOGPALETTE  pGLP  = 0;
HANDLE        hGLP;

void FreeGlobalVars()
{		// if old bitmap, delete same ...
	if( hGBM ) DeleteObject( hGBM );
	if( hGBM ) LocalFree( (HANDLE) hGBM );
	hGBM = NULL;
		// if old palette, delete same
	if( hGPal ) DeleteObject( hGPal );
	if( hGPal ) LocalFree( (HANDLE) hGPal );
	hGPal = NULL;
		// if old palette handle, delete same
	if( hGLP ) DeleteObject( hGLP );
	if( hGLP ) LocalFree( hGLP );
	hGLP = NULL;
	bmWidth = bmHeight = bmImgSize = bmScanWidth = 0;
}

int ReadBitmap( HWND hwnd, LPCSTR szFName )
{
	BITMAPFILEHEADER   bmFH;
	LPBITMAPINFOHEADER pBmIH;
	LPBITMAPINFO       pBmInfo;
	RGBQUAD   FAR    * pRGB;
	BYTE             * pBits;
	HDC			hdc;
	HANDLE		hBM, hBmInfo, hBmIH;
	HBITMAP		hBitmap;
	HPALETTE	hOldPal, hNewPal;
	WORD		PalSize, bmBitCount;
	int			hFile;
	char		szBuff[64];
	LPSTR		lpBits;
	int			i, CRes = 256;
	LONG		GetBytes, ReadBytes, BytesLeft;

		//=== clean up any existing memory objects ==============
	FreeGlobalVars();
		//=== open file ==========================================
	hFile = _lopen( szFName, OF_READ );
	if( hFile == -1 )
	{
		sprintf( szBuff, "Can't open %s", szFName );
		return( ErrorMsg( szBuff ) );
	}
		//=== read bitmap header =================================
	SetCursor( LoadCursor( NULL, IDC_WAIT ) );
	_lread( hFile, (LPSTR) &bmFH, sizeof(BITMAPFILEHEADER) );
	if( bmFH.bfType != 0x4D42 )   // if type isn't "BF" ...
		return( ErrorMsg( "Not a bitmap image" ) );
		//=== read bitmap info header ============================
	hBmIH = LocalAlloc( LMEM_FIXED, sizeof(BITMAPINFOHEADER ) );
	pBmIH = (LPBITMAPINFOHEADER) LocalLock( hBmIH );
	_lread( hFile, (LPSTR) pBmIH, sizeof(BITMAPINFOHEADER) );
	if( (WORD) pBmIH->biBitCount != 8 ) // 2^8 colors = 256
		return( ErrorMsg( "Not a 256 color bitmap"));
	if( (DWORD) pBmIH->biCompression != BI_RGB )
		return( ErrorMsg("Compressed images not accepted"));
	PalSize = (WORD)( pBmIH->biClrUsed * sizeof(RGBQUAD) );
	GlobalUnlock( hBmIH );
		//=== read palette info into bitmap info =================
	hBmInfo = LocalAlloc( LPTR, PalSize + sizeof(BITMAPINFOHEADER) );
	pBmInfo = (LPBITMAPINFO) LocalLock( hBmInfo );
	pBmInfo->bmiHeader = *pBmIH;
	LocalFree( hBmIH );          // free old memory handle
		//=== save size data for this image ======================
	bmWidth     = LOWORD( pBmInfo->bmiHeader.biWidth );
	bmHeight    = LOWORD( pBmInfo->bmiHeader.biHeight );
		//  note width and height saved as WORD, not DWORD
	bmBitCount  = (WORD)  pBmInfo->bmiHeader.biBitCount;
	bmImgSize   = (DWORD) pBmInfo->bmiHeader.biSizeImage;
	bmScanWidth = (DWORD) bmWidth;
	while( bmScanWidth % sizeof(WORD) ) bmScanWidth++;
                          // must be an even WORD size !!!
		//=== if image size not in header, calculate size ========
	if( pBmInfo->bmiHeader.biSizeImage == 0 )
	{
		bmImgSize = ( ( ( ( bmWidth * bmBitCount ) + 31 ) / 32 ) * 4 ) * bmHeight;
		pBmInfo->bmiHeader.biSizeImage = bmImgSize;
	}
		//=== now read the palette information ===================
	if( _lread( hFile, (LPSTR) pBmInfo->bmiColors, PalSize ) != PalSize )
	{
		GlobalUnlock( hBmInfo );
		LocalFree( hBmInfo );
		pBmInfo = NULL;
		FreeGlobalVars();
		return( ErrorMsg( "Palette read error" ) );
	}
		//=== allocate space for bitmap palette ==================
	pRGB = pBmInfo->bmiColors;
		//================================================//
		// check for short palette indicated by biClrUsed //
		//================================================//
	if( pBmInfo->bmiHeader.biClrUsed )
		CRes = pBmInfo->bmiHeader.biClrUsed;
	if( hGLP != NULL ) LocalFree( hGLP );
	hGLP = LocalAlloc( LPTR, sizeof(LOGPALETTE) +
					   ( ( CRes - 1 ) * sizeof(PALETTEENTRY) ) );
	pGLP = (LPLOGPALETTE) LocalLock( hGLP );
                // allocate and lock memory for palette
	pGLP->palNumEntries = CRes;       // fill in size and
	pGLP->palVersion    = 0x0300;     // version (3.0)
	for( i=0; i<CRes; i++ )           // convert colors
	{
		pGLP->palPalEntry[i].peRed   = pRGB[i].rgbRed;
		pGLP->palPalEntry[i].peGreen = pRGB[i].rgbGreen;
		pGLP->palPalEntry[i].peBlue  = pRGB[i].rgbBlue;
		pGLP->palPalEntry[i].peFlags = 0;
	}
	hNewPal = CreatePalette( pGLP );
	GlobalUnlock( hGLP );
		//=== now retrive actual image information ===============
	hBM = LocalAlloc( LPTR, bmImgSize );
	BytesLeft = ReadBytes = bmImgSize;
	pBits = (BYTE *) LocalLock( hBM );
	while( BytesLeft > 0 )
	{
		if( BytesLeft > 32767L ) GetBytes = 32768L;
							else GetBytes = BytesLeft;
         //============================================//
         // limit block reads to 32K to avoid crossing //
         // segment boundry when calling _lread        //
         //============================================//
		if( (WORD) _lread( hFile, (LPSTR) pBits, (WORD) GetBytes ) != GetBytes )
			if( ReadBytes <= 0 ) break;
			else
			{
				FreeGlobalVars();
				return( FALSE );
			}
		pBits += GetBytes;
		ReadBytes -= GetBytes;
		BytesLeft -= GetBytes;
	}
	_lclose( hFile );
		//=== make new palette active ============================
	hdc = GetDC( GetFocus() );
	hOldPal = SelectPalette( hdc, hNewPal, FALSE );
	RealizePalette( hdc );
		//=== create DIB Bitmap ==================================
	lpBits = LocalLock( hBM );
	hBitmap = CreateDIBitmap( hdc,
		 (LPBITMAPINFOHEADER) &pBmInfo->bmiHeader,
							  CBM_INIT,
							  lpBits,
			   (LPBITMAPINFO) pBmInfo,
							  DIB_RGB_COLORS );

	while( LOBYTE( LocalFlags( hBM ) ) != 0 )
		LocalUnlock( hBM );  // remember, this was locked at least twice

	if( hBitmap != NULL )
	{
		if( hGBM ) DeleteObject( hGBM );   // if exist, delete same ...
		hGBM = hBitmap;                    // save global handle
		if( hGPal ) DeleteObject( hGPal ); // if exist, delete same ...
		hGPal = hNewPal;                   // save global handle
	} 
	else 
		ErrorMsg( "Bitmap failed" );
		//=== discard global memory object =======================
	if( hOldPal ) SelectPalette( hdc, hOldPal, FALSE );
	ReleaseDC( GetFocus(), hdc );
	GlobalUnlock( hBmInfo );
	LocalFree( hBmInfo );
	pBmInfo = NULL;
	LocalFree( hBM );
	SetCursor( LoadCursor( NULL, IDC_ARROW ) );
	InvalidateRect( hwnd, NULL, TRUE );
	return(0);
}
