//========================//
//       MAPDEMO.C        //
//  256-Color Map Viewer  //
//========================//

#include <windows.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "mapdemo.h"

#ifndef __BORLANDC__
#define random(x)  ( rand() % (int)( x ) )
#endif

#define StateColors  41
#define StateCoords  13
#define AllCoords    StateCoords+3
#define BORDER       0x00000000
#define APP_ICON     "MAPDEMO"
#define APP_MENU     "MAPDEMO"
#define APP_CURSOR   "MAPDEMO"

char    szAppTitle[] = "256-Color Map Viewer",
        szAppName[]  = "MapDemo";
HANDLE  hInst;
BOOL    bReadColor;

typedef struct { char  *State;
                 short  Color;  } ColorState;

typedef struct { char  *State;
                 short  xPos;
                 short  yPos;   } CoordState;

short NewEngland[12] =
{  60,  61,  65,  67,  69,  70,
   72,  76,  80, 151, 153, 185  };

ColorState CState[StateColors] =
{	"Arizona",		  2,		"New Mexico",		  7,
	"Oklahoma",		  9,		"Georgia",			 11,
	"Oregon",		 12,		"Colorado",			 13,
	"Missouri",		 15,		"South Carolina",	 16,
	"Texas",		 17,		"Hawaii",			 18,	// 10
	"Virginia",		 21,		"Arkansas",			 23,
	"Nebraska",		 24,		"Illinois",			 27,
	"Mississippi",	 29,		"Louisana",			 35,
	"Ohio",			 39,		"California",		 40,
	"Florida",		 41,		"Idaho",			 52,	// 20
	"Michigan",		 56,		"North Carolina",	 59,
	"Kansas",		 62,		"Washington",		 65,
	"Wisconsin",	 68,		"West Virginia",	 71,
	"Minnesota",	 74,		"Kentucky",			 77, 
	"Iowa",			 79,		"Nevada",			 81,	// 30
	"Tennessee",	 83,		"Utah",				109,
	"South Dakota",	114,		"North Dakota",		121,
	"Wyoming",		142,		"Alabama",			158, 
	"Alaska",		180,		"Montana",			189,
	"Indiana",		194,		"Alaska",			248,	// 40
	"Hawaii",		255   };								// 41

CoordState Coord[AllCoords] =
{	"Connecticut",		267, 208,	"Delaware",			202, 311,
	"Maine",			330,  86,	"Maryland",			148, 305,
	"Maryland",			 79, 306,	"Massachusetts",	279, 184,
	"New Hampshire",	276, 144,	"New Jersey",		209, 253,
	"New York",			181, 178,	"New York",			258, 237,
	"Pennsylvania",		119, 256,	"Rhode Island",		300, 201,
	"Vermont",			238, 132,	"RETURN",			 14, 337,
	"RETURN",			331, 290,	"RETURN",			122,  81  };

int ErrorMsg( char *Error )
{
	MessageBox( GetFocus(), Error, "Notice", MB_ICONASTERISK | MB_OK );
	return(0);
}

#include "dibitmap.i"

void LocationMsg( char *State )
{
	char szBuff[80];

	wsprintf( szBuff, "Welcome to the great state of %s", State );
	MessageBox( GetFocus(), szBuff, "Location", MB_ICONEXCLAMATION | MB_OK );                        
	return;
}

//=== use color matching for the USMAP01 image ===========

void ColorCheckMap( HWND hwnd, WORD xCoord, WORD yCoord )
{
	HDC      hdc;
	DWORD    RColor;
	WORD     SColor;
	int      i;

	if( ( xCoord >= ( bmWidth - 1  ) ) || ( yCoord >= ( bmHeight - 1 ) ) )
	{
		MessageBeep( MB_ICONASTERISK );
		ErrorMsg( "Point selected must\nbe within map bounds" );
		return;
	}
	hdc = GetDC( hwnd );
	RColor = GetPixel( hdc, xCoord, yCoord );
			// need RGB palette-relative color value
	ReleaseDC( hwnd, hdc );
	if( RColor == BORDER )
	{
		MessageBeep( MB_ICONASTERISK );
		ErrorMsg( "Select a point within\na state boundry ..." );
		return;
	}
	LocalLock( hGLP );
	LocalLock( hGPal );
			// lock and create palette for reference
	hGPal = CreatePalette( pGLP );
	if( hGPal == NULL ) ErrorMsg( "Palette not found!" );
			// make palette active for device context
	RealizePalette( hdc );
			// get palette index for comparison
	SColor = GetNearestPaletteIndex( hGPal, (COLORREF) RColor );
			// unlock everything but don't delete palette
	LocalUnlock( hGLP );
	LocalUnlock( hGPal );
	for( i=0; i<12; i++ )
		if( SColor == NewEngland[i] )
				// if any New England state, switch maps
			PostMessage( hwnd, WM_COMMAND, IDM_MAP2, 0L );
	for( i=0; i<=StateColors; i++ )
		if( SColor == CState[i].Color )
			LocationMsg( CState[i].State );
	MessageBeep( MB_ICONASTERISK );
}

//=== use drunkard's walk algorithm and coordinate =======
//=== matching for the USMAP02 image =====================

void CoordCheckMap( HWND hwnd, WORD xCoord, WORD yCoord )
{
	BOOL     Done = FALSE, Reverse;
	HDC      hdc;
	WORD     i;
	int      j, k, x, y;
	HCURSOR  hOrgCur;

	if( ( xCoord >= ( bmWidth - 1  ) ) || ( yCoord >= ( bmHeight - 1 ) ) )
	{
		MessageBeep( MB_ICONASTERISK );
		ErrorMsg( "Point selected must\nbe within map bounds" );
		return;
	}
#ifdef __BORLANDC__
	randomize();
#endif
	hOrgCur = SetCursor( LoadCursor( NULL, IDC_WAIT ) );
	hdc = GetDC( hwnd );
	x = random(3)-1;
	y = random(3)-1;
	do
	{
		for( i=0; i<AllCoords; i++ )
		{
			if( ( abs( xCoord - Coord[i].xPos ) +
				  abs( yCoord - Coord[i].yPos ) ) < 10 )
			{
				if( i < StateCoords )
					LocationMsg( Coord[i].State );
				else 
					PostMessage( hwnd, WM_COMMAND, IDM_MAP1, 0L );
				Done = TRUE;
		}	}
		Reverse = FALSE;
		if( ( xCoord >= bmWidth-1  ) || ( xCoord <= 1 ) ||
			( yCoord >= bmHeight-1 ) || ( yCoord <= 1 ) ) Reverse = TRUE;
		else
			for( j=-1; j<2; j++ )
				for( k=-1; k<2; k++ )
					if( GetPixel( hdc, xCoord+j, yCoord+k ) == BORDER )
						Reverse = TRUE;
		if( Reverse )
		{
			x *= -1;
			y *= -1;
		}
		else
		if( ! random(10) )
		{
			x = random(3)-1;
			y = random(3)-1;
		}
//=== option to trace random walk algorithm =========
		SetPixel( hdc, xCoord, yCoord, 0x00FFFFFF );
//===================================================
		xCoord += x;
		yCoord += y;
	}
	while( ! Done );
	ReleaseDC( hwnd, hdc );
	SetCursor( hOrgCur );
	return;
}

void ResizeWindow( HWND hwnd )
{
	RECT  rMajor, rMinor;
	int   wWidth, wHeight;

	GetWindowRect( hwnd, &rMajor );
	GetClientRect( hwnd, &rMinor );
	wWidth  = ( rMajor.right - rMajor.left ) -
			  ( rMinor.right - rMinor.left ) + bmWidth;
	wHeight = ( rMajor.bottom - rMajor.top ) -
              ( rMinor.bottom - rMinor.top ) + bmHeight;
	MoveWindow( hwnd, 0, 0, wWidth, wHeight, TRUE );
}

#ifdef __BORLANDC__
#pragma argsused
#endif

BOOL APIENTRY AboutProc( HWND hDlg,   UINT msg,
                         UINT wParam, LONG lParam )
{
	switch( msg )
	{
		case WM_INITDIALOG:  return( TRUE );
		case WM_COMMAND:
			switch( LOWORD( wParam ) )
			{
				case IDOK: EndDialog( hDlg, TRUE );	return( TRUE );
				default:							return( TRUE );
	}		}
	return( FALSE );
}

long APIENTRY WndProc( HWND hwnd,   UINT msg,
                       UINT wParam, LONG lParam )
{
	HDC           hdc, hdcMem;
	PAINTSTRUCT   ps;
	HBITMAP       hBMOld;
	HPALETTE      hPalOld;

	switch( msg )
	{
		case WM_CREATE:
			PostMessage( hwnd, WM_COMMAND, IDM_MAP1, 0L );
			break;

		case WM_COMMAND:
			switch( LOWORD( wParam ) )
			{
				case IDM_MAP1:
					ReadBitmap( hwnd, "USMAP01.BMP" );
					bReadColor = TRUE;
					ResizeWindow( hwnd );
					break;

				case IDM_MAP2:
					ReadBitmap( hwnd, "USMAP02.BMP" );
					bReadColor = FALSE;
					ResizeWindow( hwnd );
					break;

				case IDM_ABOUT:
					DialogBox( hInst, "ABOUT", hwnd, AboutProc );
					break;

				case IDM_EXIT:
					PostMessage( hwnd, WM_CLOSE, 0, 0L );
					break;

				default:
					return( DefWindowProc( hwnd, msg, wParam, lParam ) );
			}
			break;

		case WM_LBUTTONDOWN:
			if( bReadColor )
				ColorCheckMap( hwnd, LOWORD( lParam ), HIWORD( lParam ) );
			else
				CoordCheckMap( hwnd, LOWORD( lParam ), HIWORD( lParam ) );
			break;

		case WM_PAINT:
			InvalidateRect( hwnd, NULL, TRUE );
			hdc = BeginPaint( hwnd, &ps );
			if( hGBM )
			{
				hdcMem = CreateCompatibleDC( hdc );
				hBMOld = SelectObject( hdcMem, hGBM );
				if( hGPal )
					hPalOld = SelectPalette( hdc, hGPal, FALSE );
				BitBlt( hdc, 0, 0, bmWidth, bmHeight,
					    hdcMem, 0, 0, SRCCOPY );
				if( hGPal )
					SelectPalette( hdc, hPalOld, FALSE );
				SelectObject( hdcMem, hBMOld );
				DeleteDC( hdcMem );
			}
			EndPaint( hwnd, &ps );
			break;

		case WM_DESTROY:
			FreeGlobalVars();
			PostQuitMessage(0);
			break;

		default:	return( DefWindowProc( hwnd, msg, wParam, lParam ) );
	}
	return( TRUE );
}

#include "template.i"

#ifdef __BORLANDC__
#pragma argsused
#endif

int APIENTRY WinMain( HANDLE hInstance, HANDLE hPrevInstance,
                      LPSTR  lpCmdLine, int    nCmdShow    )
{
	MSG   msg;

	if( ! hPrevInstance )					// are any other instances running?
		if( ! InitApplication( hInstance ) ) 
			return( FALSE );
	if( ! InitInstance( hInstance, nCmdShow ) ) 
		return (FALSE);
	while( GetMessage( &msg,				// message structure            
					   NULL,				// handle of window addressed   
					   0,					// lowest message to examine      
					   0 ) )				// highest message to examine   
	{
		TranslateMessage( &msg );			// translate virtual key codes
		DispatchMessage( &msg );			// dispatch message to window 
	}
	return( msg.wParam );
#ifndef __BORLANDC__
	UNREFERENCED_PARAMETER( lpCmdLine );	// prevents unused para warning
#endif										//    message from compiler
}
