//======================//
//      Forest.C        //
//  Simulation Example  //
//======================//

#include <windows.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include "forest.h"

#define APP_ICON  "Forest"
//======================================================//
//  if a null icon is assigned, the territory becomes   //
//  the map and the icon appears as a miniature of the  //
//  Forest World display                                //
//======================================================//
#define APP_MENU  "Forest"
// note: no cursor assignment
#define  szCaption  "Forest World -- Year: %d"
#define  GRID  100
#define  xOfs  4
#define  yOfs  4
#define  FLAMABILITY  10		// increase to decrease burn rates

#ifndef __BORLANDC__
#define random(x)  ( rand() % (int)( x ) )
#endif

HANDLE hInst;
BYTE   Acres[GRID][GRID], NewAC[GRID][GRID], ActiveTimer;
int    nYears = 0, nDays, nWind;
char   szAppTitle[] = "Forest Simulation";
char   szAppName[]  = "Forest";
static COLORREF cColor[16] =
{
	DampSoil,	DrySoil,	NewGrowth,	Green1,
	Green2,		Green3,		Mature,		Green4,
	Green5,		Green6,		Green7,		OldGrowth,
	Burning,	Embers,		BurnOut,	Bare
};

char * lpszCaptions[16] = 
{
	"Damp Soil",		"Dry Soil",			"New Growth 1",		"New Growth 2",
	"New Growth 3",		"New Growth 4",		"Mature Trees 1",	"Mature Trees 2",
	"Mature Trees 3",	"Mature Trees 4",	"Mature Trees 5",	"Old Growth",
	"Burning",			"Embers",			"Burnout",			"Bare Ashes" 
};

int CheckPos( int nPos )
{
	if( nPos < 0 )     nPos += GRID;
	if( nPos >= GRID ) nPos -= GRID;
	return( nPos );
}

void AgeWorld()
{
	int  i, j;

	for( i=0; i<GRID; i++ )
		for( j=0; j<GRID; j++ )
		{
			NewAC[i][j] = Acres[i][j];
			switch( Acres[i][j] )
			{
				case iNewGrowth:
				case iGreen1:   case iGreen2:
				case iGreen3:   case iMature:
				case iGreen4:   case iGreen5:
				case iGreen6:   case iGreen7:
					if( ! random(5) )
						NewAC[i][j] = Acres[i][j] + 1;		break;
				case iOldGrowth:
					if( ! random(199) )
						NewAC[i][j] = iBare;				break;
				case iBurning:
					NewAC[i][j] = iEmbers;					break;
				case iEmbers:
					NewAC[i][j] = iBurnOut;					break;
				case iBurnOut:
					NewAC[i][j] = iBare;					break;
				case iBare:
					NewAC[i][j] = iDampSoil + random(2);	break;
}		}	}

void PropogateTrees()
{
	int  i, j, x, y;

	for( i=0; i<GRID; i++ )
		for( j=0; j<GRID; j++ )
			if( ( NewAC[i][j] >= iMature ) &&
				( NewAC[i][j] < iBurning ) &&
				( random(3) ) )
			{
				x = CheckPos( i + ( random(7) - 3 ) );
				y = CheckPos( j + ( random(7) - 3 ) );
				if( ( NewAC[x][y] == iDampSoil ) ||
					( NewAC[x][y] == iDrySoil )  ||
					( NewAC[x][y] == iBare   ) )
					NewAC[x][y] = iNewGrowth;
}			}

void StepForest()
{
	int  i, j;

	for( i=0; i<GRID; i++ )
		for( j=0; j<GRID; j++ )
			Acres[i][j] = NewAC[i][j];
}

void InitBurns( HWND hwnd )
{
	int  x, y, NoChance = 20;

	x = random( GRID );
	y = random( GRID );
	if( random( iOldGrowth + NoChance ) <= (int) Acres[x][y] )
	{
		nDays = 0;
		nWind = random(4);
		Acres[x][y] = iBurning;
		PostMessage( hwnd, WM_COMMAND, IDM_STOPGROWTH, 0L );
		PostMessage( hwnd, WM_COMMAND, IDM_STARTBURNS, 0L );
	}
}

void IgniteTrees( int xPos, int yPos )
{
	xPos = CheckPos( xPos );
	yPos = CheckPos( yPos );
	if( ( Acres[xPos][yPos] >= iGreen1 ) &&
		( Acres[xPos][yPos] <= iOldGrowth ) &&
		( Acres[xPos][yPos] > (BYTE) random( iOldGrowth * FLAMABILITY ) ) )
		Acres[xPos][yPos] = iBurning;
}

int TrackFire()
{
	static int  i, j, x, y, Burns, WindSpeed;
	char szBuff[80], szWind[6];

	WindSpeed = random(4)+2;
	Burns = 0;
	switch( nWind )
	{
		case 0: strcpy( szWind, "North" ); break;
		case 1: strcpy( szWind, "West" );  break;
		case 2: strcpy( szWind, "South" ); break;
		case 3: strcpy( szWind, "East" );
	}
	wsprintf( szBuff, "Year: %d  Burn: %d  Wind from %s at %d",
			  nYears, ++nDays, szWind, WindSpeed );
	SetWindowText( GetFocus(), szBuff );
	switch( nWind )
	{
		case 0:     // wind from North
			SetWindowText( GetFocus(), szBuff );
			for( j=GRID-1; j>=0; j-- )
				for( i=GRID-1; i>=0; i-- )
				{
					if( ( Acres[i][j] == iBurning ) ||
						( Acres[i][j] == iEmbers  ) )
					{
						Burns++;
						for( y=1; y<WindSpeed; y++ )
							for( x=-y; x<=y; x++ )
								IgniteTrees( i+x, j+y );
					}
					if( Acres[i][j] == iEmbers )
						Acres[i][j] = iBurnOut;
					if( ( nDays > random(20)+5 ) &&
						( Acres[i][j] == iBurning ) )
						Acres[i][j] = iEmbers;
				}
				break;

		case 1:     // wind from West
			for( i=GRID-1; i>=0; i-- )
				for( j=GRID-1; j>=0; j-- )
				{
					if( ( Acres[i][j] == iBurning ) ||
						( Acres[i][j] == iEmbers  ) )
					{
						Burns++;
						for( x=1; x<WindSpeed; x++ )
							for( y=-x; y<=x; y++ )
								IgniteTrees( i+x, j+y );
					}
					if( Acres[i][j] == iEmbers )
						Acres[i][j] = iBurnOut;
					if( ( nDays > random(20)+5 ) &&
						( Acres[i][j] == iBurning ) )
						Acres[i][j] = iEmbers;
				}
				break;

		case 2:     // wind from South
			for( j=0; j<GRID; j++ )
				for( i=0; i<GRID; i++ )
				{
					if( ( Acres[i][j] == iBurning ) ||
						( Acres[i][j] == iEmbers  ) )
					{
						Burns++;
						for( y=1; y<WindSpeed; y++ )
							for( x=-y; x<=y; x++ )
								IgniteTrees( i-x, j-y );
					}
					if( Acres[i][j] == iEmbers )
						Acres[i][j] = iBurnOut;
					if( ( nDays > random(20)+5 ) &&
						( Acres[i][j] == iBurning ) )
						Acres[i][j] = iEmbers;
				}
				break;

		case 3:     // wind from East
			for( i=0; i<GRID; i++ )
				for( j=0; j<GRID; j++ )
				{
					if( ( Acres[i][j] == iBurning ) ||
						( Acres[i][j] == iEmbers  ) )
					{
						Burns++;
						for( x=1; x<WindSpeed; x++ )
							for( y=-x; y<=x; y++ )
								IgniteTrees( i-x, j-y );
					}
					if( Acres[i][j] == iEmbers )
						Acres[i][j] = iBurnOut;
					if( ( nDays > random(20)+5 ) &&
						( Acres[i][j] == iBurning ) )
						Acres[i][j] = iEmbers;
	}			}
	return( Burns );  // is anything still burning?
}

long FAR PASCAL WndProc( HWND hwnd,   UINT msg,
                         UINT wParam, LONG lParam )
{
	static int   i, j, xPos, yPos;
	char         szBuff[64];
	HDC          hdc;
	HBRUSH       hBrush[16];
	RECT         rect;
	PAINTSTRUCT  ps;

	switch( msg )
	{
		case WM_CREATE:
#ifdef __BORLANDC__
			randomize();
#endif
			ActiveTimer = 0;
			wsprintf( szBuff, szCaption, nYears );
			SetWindowText( hwnd, szBuff );
			for( i=0; i<GRID; i++ )         // init world as
				for( j=0; j<GRID; j++ )      // bare ground
	               Acres[i][j] = random(2);

//=== optional code to show color settings ============//
//      for( i=0; i<GRID; i++ )                        //
//         for( j=0; j<GRID; j++ )                     //
//            Acres[i][j] = (int)((i*16)/GRID);        //
//=====================================================//

			break;

		case WM_COMMAND:
			switch( LOWORD( wParam ) )
			{
				case IDM_STARTTIMER:
					EnableMenuItem( GetMenu(hwnd), IDM_STARTTIMER, MF_GRAYED );
					EnableMenuItem( GetMenu(hwnd), IDM_STOPTIMER,  MF_ENABLED );
					switch( ActiveTimer )
					{
						case ID_GROWTH:
							PostMessage( hwnd, WM_COMMAND, IDM_STARTGROWTH, 0L );
							break;

						case ID_FIRES:
							PostMessage( hwnd, WM_COMMAND, IDM_STARTBURNS, 0L );
							break;
					}
					break;

				case IDM_STOPTIMER:
					EnableMenuItem( GetMenu(hwnd), IDM_STOPTIMER,  MF_GRAYED );
					EnableMenuItem( GetMenu(hwnd), IDM_STARTTIMER, MF_ENABLED );
					switch( ActiveTimer )
					{
						case ID_GROWTH:
							PostMessage( hwnd, WM_COMMAND, IDM_STOPGROWTH, 0L );
							break;

						case ID_FIRES:
							PostMessage( hwnd, WM_COMMAND, IDM_STOPBURNS, 0L );
							break;
					}
					break;

				case IDM_STARTGROWTH:
					SetTimer( hwnd, ID_GROWTH, 1000, NULL );
					ActiveTimer = ID_GROWTH;
					break;

				case IDM_STOPGROWTH:
					KillTimer( hwnd, ID_GROWTH );
					break;

				case IDM_STARTBURNS:
					SetTimer( hwnd, ID_FIRES, 100, NULL );
					ActiveTimer = ID_FIRES;
					break;

				case IDM_STOPBURNS:
					KillTimer( hwnd, ID_FIRES );
					break;

				case IDM_RESEED:
#ifdef __BORLANDC__
					randomize();
#else
					srand( (unsigned)time( NULL ) );
#endif
					nYears = 0;
					for( i=0; i<100; i++ )
					{
						xPos = random(GRID);
						yPos = random(GRID);
						Acres[xPos][yPos] = iNewGrowth;
					}
					EnableMenuItem( GetMenu(hwnd), IDM_STOPTIMER, MF_ENABLED );
					PostMessage( hwnd, WM_COMMAND, IDM_STARTGROWTH, 0L );
					InvalidateRect( hwnd, NULL, FALSE );
					break;

				default:
					return( DefWindowProc( hwnd, msg, wParam, lParam ) );
			}
			break;

		case WM_PAINT:
			hdc = BeginPaint( hwnd, &ps );
			GetClientRect( hwnd, &rect );
			SetMapMode( hdc, MM_ISOTROPIC );
			SetWindowOrgEx( hdc, 0, 0, NULL );
			SetWindowExtEx( hdc, ( 110 * GRID ) / rect.right,
								 ( 110 * GRID ) / rect.bottom, NULL );
			SetViewportExtEx( hdc, GRID, GRID, NULL );
			for( i=0; i<16; i++ )
				hBrush[i] = CreateSolidBrush( cColor[i] );
			for( i=0; i<GRID; i++ )
				for( j=0; j<GRID; j++ )
				{
					SetRect( &rect, i + xOfs,     j + yOfs,
							 i + xOfs + 1, j + yOfs + 1 );
					FillRect( hdc, &rect, hBrush[Acres[i][j]]);
					//=== use for SetPixel versus FillRect functions =========
					// SetPixel( hdc, i, j, cColor[Acres[i][j]] );
					//========================================================
				}
			for( i=0; i<16; i++ )
			{
				SetRect( &rect, GRID + 4 + xOfs, ( 5 * i ) + yOfs,
						 GRID + 7 + xOfs, ( 5 * ( i + 1 ) ) + yOfs - 1 );
				FillRect( hdc, &rect, hBrush[i] );
				TextOut( hdc, GRID + 8 + xOfs, ( i * 5 ) + yOfs,
						 lpszCaptions[i], strlen( lpszCaptions[i] ) );
			}
			for( i=0; i<16; i++ )
				DeleteObject( hBrush[i] );
			EndPaint( hwnd, &ps );
			break;

		case WM_TIMER:
			switch( LOWORD( wParam ) )
			{
				case ID_GROWTH:
					wsprintf( szBuff, szCaption, nYears++ );
					SetWindowText( hwnd, szBuff );
					AgeWorld();
					PropogateTrees();
					StepForest();
					InitBurns( hwnd );
					InvalidateRect( hwnd, NULL, FALSE );
					break;

				case ID_FIRES:
//					MessageBeep( 0 );
					if( ! random(4) ) nWind = random(4);
					if( ! TrackFire() )
					{
						PostMessage( hwnd, WM_COMMAND, IDM_STOPBURNS, 0L );
						PostMessage( hwnd, WM_COMMAND, IDM_STARTGROWTH, 0L );
					}
					InvalidateRect( hwnd, NULL, FALSE );
					break;

				default:
					return( DefWindowProc( hwnd, msg, wParam, lParam ) );
			}
			break;

		case WM_DESTROY:
			PostMessage( hwnd, WM_COMMAND, IDM_STOPTIMER, 0L );
			PostQuitMessage(0);
			break;

		default:
			return( DefWindowProc( hwnd, msg, wParam, lParam ) );
	}
	return 0;
}

#include "template.i"

#ifdef __BORLANDC__
#pragma argsused
#endif

int APIENTRY WinMain( HANDLE hInstance, HANDLE hPrevInstance,
                      LPSTR  lpCmdLine, int    nCmdShow    )
{
	MSG   msg;

	if( ! hPrevInstance )
		if( ! InitApplication( hInstance ) )
			return( FALSE );
	if( ! InitInstance( hInstance, nCmdShow ) )
		return (FALSE);
	while( GetMessage( &msg, NULL, 0, 0 ) )
	{
		TranslateMessage( &msg );
		DispatchMessage( &msg );
	}
	return( msg.wParam );
#ifndef __BORLANDC__
	UNREFERENCED_PARAMETER( lpCmdLine );
#endif
}
