// GrayImageView.cpp : implementation of the CGrayImageView class
//

#include "stdafx.h"
#include "GrayImage.h"

#include "GrayImageDoc.h"
#include "GrayImageView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////
// CGrayImageView

IMPLEMENT_DYNCREATE(CGrayImageView, CView)

BEGIN_MESSAGE_MAP(CGrayImageView, CView)
	//{{AFX_MSG_MAP(CGrayImageView)
	ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////
// CGrayImageView construction/destruction

CGrayImageView::CGrayImageView()
{
	m_hBitmap = NULL;
	m_pPal	  = NULL;
	m_pLogPal = NULL;
	m_bBitmap = FALSE;
	m_hBmInfo = NULL;
}

CGrayImageView::~CGrayImageView()
{
	FreeGlobalVars();
}

BOOL CGrayImageView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs
	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////
// CGrayImageView drawing

void CGrayImageView::OnDraw(CDC* pDC)
{
	CGrayImageDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CDC		*pDCMem, *pDCSrc;
	UINT	 i, j, m, n;
	int		 nBitPxl, nPlanes, nGrayWd, nGrayHt;
	BYTE	 rVal, gVal, bVal, Gray;
	DWORD    Color;
	HBITMAP	 hTargetBM;
	WORD	 Mask,
			 GrayPal[16] = { 0x0000, 0x0400, 0x0401, 0x0501, // gray scale masks
							 0x0505, 0x0525, 0xA425, 0xA5A5,
							 0xA7A5, 0xE5B5, 0xF5B6, 0xF5F5,
							 0xF5FD, 0xF7FD, 0xF7FF, 0xFFFF };
	if( m_hBitmap )
	{
		nBitPxl = pDC->GetDeviceCaps( BITSPIXEL );
		nPlanes = pDC->GetDeviceCaps( PLANES );
		pDCMem = new CDC();
		pDCMem->CreateCompatibleDC( pDC );
		if( ( nBitPxl * nPlanes ) > 1 )
		{			//=== we assume that the device context supports color ======
			CPalette *pOldPalette;

			if( m_pPal ) 
			{
				pOldPalette = pDC->SelectPalette( m_pPal, FALSE );
				pDC->RealizePalette();
			}
			if( pDCMem->SelectObject( m_hBitmap ) )
					//=== copy the bitmap to the screen ==(NORMAL)===============
				pDC->BitBlt( 0, 0, m_bmWidth, m_bmHeight, pDCMem, 0, 0, SRCCOPY );
			else
					//===	since the bitmap was not selected to memory context ===
					//=== we assume this is probably a printer context, ergo ...
				SetDIBitsToDevice( pDC->m_hDC, 0, 0, m_bmWidth, m_bmHeight,
								   0, 0, 0, m_bmHeight,
								   m_pBits, m_pBmInfo, DIB_RGB_COLORS );
			if( m_pPal )
			{
				pDC->SelectPalette( pOldPalette, FALSE );
				pDC->RealizePalette();
			}
		}
		else
		{		// if this is a monochrome device - i.e. a printer -
				//		get the max size supported by the device
			nGrayWd = min( (int)(4 * m_bmWidth),  pDC->GetDeviceCaps( HORZRES ) );
			nGrayHt = min( (int)(4 * m_bmHeight), pDC->GetDeviceCaps( VERTRES ) );
				//		and create a gray-scaled bitmap to print
			hTargetBM = CreateBitmap( nGrayWd, nGrayHt, nPlanes, nBitPxl, NULL );
			if( ! hTargetBM )
			{
				ErrorMsg( "Bitmap creation error" );
				return;
			}
			pDCMem->SelectObject( hTargetBM );
				// create a device context compatible with the
				//		(color) display context, not the printer context
			pDCSrc = new CDC();
			pDCSrc->CreateCompatibleDC( GetDC() );
				//		but select the gray-scaled bitmap to the context
			pDCSrc->SelectObject( m_hBitmap );
			for( i=0; i<m_bmHeight; i++ )
				for( j=0; j<m_bmWidth; j++ )
				{
					Color = pDCSrc->GetPixel( j, i );
					rVal = (unsigned char)( LOBYTE( HIWORD( Color ) ) );
					gVal = (unsigned char)( HIBYTE( LOWORD( Color ) ) );
					bVal = (unsigned char)( LOBYTE( LOWORD( Color ) ) );
						//=== convert color to gray intensity =================
					if( theApp.m_bTrueGray )
						Gray = (BYTE)( (UINT)(float)( ( rVal * 0.30 ) / 16 ) +
									   (UINT)(float)( ( gVal * 0.59 ) / 16 ) +
									   (UINT)(float)( ( bVal * 0.11 ) / 16 ) );
					else
						Gray = (BYTE)( ( rVal + gVal + bVal ) / 48 );
					//========================================================//	
					// for printing, the TRUEGRAY algorithm produces a rather //
					// dark image. Instead, use unweighted intensity or       //
					//	include a logarithmic scale to keep blacks black and   //
					//	whites white but shift most colors toward the light    //
					// end of the scale                                       //
					//========================================================//
					Mask = GrayPal[Gray];
						//=== write gray mask to color bitmap context =========
					for( m=0; m<4; m++ )
						for( n=0; n<4; n++ )
						{
							if( ( Mask >> ((m*4)+n) ) & 0x0001 )
								pDCMem->SetPixel( (j*4)+m, (i*4)+n, 0x00FFFFFF );
							else
								pDCMem->SetPixel( (j*4)+m, (i*4)+n, 0x00000000 );
						}
				}
				//=== now copy from color context to printer context =====
			pDC->BitBlt( 0, 0, nGrayWd, nGrayHt, pDCMem, 0, 0, SRCCOPY );
				//===	the result is printed as black and white ===========
			DeleteObject( hTargetBM );
			delete pDCSrc;
		}	
		delete pDCMem;
	}
}

/////////////////////////////////////////////////////////////////////////
// CGrayImageView printing

BOOL CGrayImageView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CGrayImageView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CGrayImageView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////
// CGrayImageView diagnostics

#ifdef _DEBUG
void CGrayImageView::AssertValid() const
{
	CView::AssertValid();
}

void CGrayImageView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CGrayImageDoc* CGrayImageView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CGrayImageDoc)));
	return (CGrayImageDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////
// CGrayImageView message handlers

void CGrayImageView::OnFileOpen() 
{
	CFileDialog cfDlg( TRUE );
	
	cfDlg.m_ofn.lpstrFilter = "Bitmap images\0*.bmp;*.dib\0\0";
//	cfDlg.m_ofn.lpstrInitialDir = "..";
	if( cfDlg.DoModal() )
	{
		m_csFileName = cfDlg.GetFileName();
		ReadBitmap( cfDlg.GetPathName() );
		Invalidate();
	}
}

void CGrayImageView::FreeGlobalVars()
{
      // if old bitmap, delete same ...
   if( m_hBitmap ) DeleteObject( m_hBitmap );
   if( m_hBitmap ) LocalFree( (HANDLE) m_hBitmap );
   m_hBitmap = NULL;
      // if old palette, delete same
   if( m_pPal ) m_pPal->DeleteObject();
   m_pPal = NULL;
      // if old palette handle, delete same
   if( m_pLogPal ) DeleteObject( m_pLogPal );
   if( m_pLogPal ) LocalFree( m_pLogPal );
   m_pLogPal = NULL;
   m_bmWidth = m_bmHeight = m_bmImgSize = m_bmScanWidth = 0;
}

BOOL CGrayImageView::ReadBitmap( CString csFName )
{
	BITMAPFILEHEADER   bmFH;
	LPBITMAPINFOHEADER pBmIH;
	RGBQUAD   FAR    * pRGB;
	CDC				 * pDC;
	HANDLE			   hBM, hBmIH; 
	HBITMAP			   hBitmap;
	WORD			   PalSize, bmBitCount;
	LPSTR			   lpBits;
	int				   i, CRes = 256;

	FreeGlobalVars();		//=== clean up any existing memory objects ===
	TRY						//=== open the bitmap file ===================
	{
		CFile cFile( csFName, CFile::modeRead | CFile::typeBinary );
		SetCursor( LoadCursor( NULL, IDC_WAIT ) );
			//=== read bitmap header ===
		cFile.Read( (LPSTR) &bmFH, sizeof(BITMAPFILEHEADER) );
		if( bmFH.bfType != 0x4D42 )		   // if type isn't "BF" ...
			return( ErrorMsg( "Not a bitmap image" ) );
			//=== read bitmap info header ===
		hBmIH = LocalAlloc( LMEM_FIXED, sizeof(BITMAPINFOHEADER ) );
		pBmIH = (LPBITMAPINFOHEADER) LocalLock( hBmIH );
		cFile.Read( (LPSTR) pBmIH, sizeof(BITMAPINFOHEADER) );
		if( (WORD) pBmIH->biBitCount != 8 ) // 2^8 colors = 256
			return( ErrorMsg( "Not a 256 color bitmap"));
		if( (DWORD) pBmIH->biCompression != BI_RGB )
			return( ErrorMsg("Compressed images not accepted"));
		PalSize = (WORD)( pBmIH->biClrUsed * sizeof(RGBQUAD) );
		GlobalUnlock( hBmIH );
		if( m_hBmInfo )		//=== free this if it already exists ===
		{
			GlobalUnlock( m_hBmInfo );
			LocalFree( m_hBmInfo );
			m_pBmInfo = NULL;
		}
			//=== read palette info into bitmap info ===
		m_hBmInfo = LocalAlloc( LPTR, PalSize + sizeof(BITMAPINFOHEADER) );
		m_pBmInfo = (LPBITMAPINFO) LocalLock( m_hBmInfo );
		m_pBmInfo->bmiHeader = *pBmIH;
		LocalFree( hBmIH );          // free old memory handle
			//=== save size data for this image ===
		m_bmWidth     = LOWORD( m_pBmInfo->bmiHeader.biWidth );
		m_bmHeight    = LOWORD( m_pBmInfo->bmiHeader.biHeight );
			//=== note width and height saved as WORD, not DWORD ===
		bmBitCount  = (WORD)  m_pBmInfo->bmiHeader.biBitCount;
		m_bmImgSize   = (DWORD) m_pBmInfo->bmiHeader.biSizeImage;
		m_bmScanWidth = (DWORD) m_bmWidth;
		while( m_bmScanWidth % sizeof(WORD) ) 
			m_bmScanWidth++;					  // must be an even WORD size !!!
			//=== if image size not in header, calculate size ===
		if( m_pBmInfo->bmiHeader.biSizeImage == 0 )
		{
			m_bmImgSize = (((( m_bmWidth * bmBitCount ) + 31 ) / 32 ) * 4 )
						  * m_bmHeight;
			m_pBmInfo->bmiHeader.biSizeImage = m_bmImgSize;
		}
			//=== now read the palette information ===
		if( cFile.Read( (LPSTR) m_pBmInfo->bmiColors, PalSize ) != PalSize )
		{
			GlobalUnlock( m_hBmInfo );
			LocalFree( m_hBmInfo );
			m_pBmInfo = NULL;
			FreeGlobalVars();
			return( ErrorMsg( "Palette read error" ) );
		}
			//=== allocate space for bitmap palette ===
		pRGB = m_pBmInfo->bmiColors;
			//================================================//
			// check for short palette indicated by biClrUsed //
			//================================================//
		if( m_pBmInfo->bmiHeader.biClrUsed )
			CRes = m_pBmInfo->bmiHeader.biClrUsed;
		if( m_pLogPal != NULL ) LocalFree( m_pLogPal );
		m_pLogPal = (tagLOGPALETTE *) LocalAlloc( LPTR, sizeof(LOGPALETTE) +
					( ( CRes - 1 ) * sizeof(PALETTEENTRY) ) );
		m_pLogPal = (LPLOGPALETTE) LocalLock( m_pLogPal );
			//=== allocate and lock memory for palette ===
		m_pLogPal->palNumEntries = CRes;       // fill in size and
		m_pLogPal->palVersion    = 0x0300;     // version (3.0)
		for( i=0; i<CRes; i++ )						// convert colors
		{
			m_pLogPal->palPalEntry[i].peRed   = pRGB[i].rgbRed;
			m_pLogPal->palPalEntry[i].peGreen = pRGB[i].rgbGreen;
			m_pLogPal->palPalEntry[i].peBlue  = pRGB[i].rgbBlue;
			m_pLogPal->palPalEntry[i].peFlags = PC_NOCOLLAPSE;
		}


		if( m_pPal ) m_pPal->DeleteObject();
		m_pPal = new CPalette();
		m_pPal->CreatePalette( m_pLogPal );
		GlobalUnlock( m_pLogPal );
			//=== now retrive actual image information ===
		hBM = LocalAlloc( LPTR, m_bmImgSize );
		m_pBits = (BYTE *) LocalLock( hBM );
		cFile.Read( m_pBits, m_bmImgSize );
		cFile.Close();
		pDC = GetDC();
			//=== make new palette active ===
		CPalette * pOldPal = pDC->SelectPalette( m_pPal, FALSE );
		pDC->RealizePalette();
			//=== create DIB Bitmap ===
		lpBits = (char*) LocalLock( hBM );
		hBitmap = CreateDIBitmap( pDC->m_hDC,
			 (LPBITMAPINFOHEADER) &m_pBmInfo->bmiHeader,
								  CBM_INIT,
								  lpBits,
				   (LPBITMAPINFO) m_pBmInfo,
								  DIB_RGB_COLORS );
		while( LOBYTE( LocalFlags( hBM ) ) != 0 )
			 LocalUnlock( hBM );  // remember, this was locked at least twice
		if( hBitmap != NULL )
		{
			if( m_hBitmap )
				DeleteObject( m_hBitmap );			// if exist, delete same ...
			m_hBitmap = hBitmap;						// then save global handle
		} 
		else 
			ErrorMsg( "Bitmap failed" );
		ReleaseDC( pDC );

		//=== we don't free hBM because it is now the location pointed 
		//=== to by m_hBitmap and m_hBitmap will free the memory when 
		//=== needed the hBM handle has strictly local scope and will
		//=== vanish by itself without action ...

		SetCursor( LoadCursor( NULL, IDC_ARROW ) );
		InvalidateRect( NULL, TRUE );
	}
	CATCH( CFileException, e )
	{
		#ifdef _DEBUG
			afxDump << "File access failed: " << e->m_cause << "\n";
		#endif
		return FALSE;
	}
	END_CATCH
	return TRUE;
}

BOOL CGrayImageView::ErrorMsg( char* Error )
{
   MessageBox( Error, "NOTICE", MB_ICONASTERISK | MB_OK );
   return FALSE;
}
