//=======================//
//        ClipBd.C       //
//  C++ Windows Drawing  //
//  creating a metafile  //
//=======================//

#include <windows.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "clipbd.h"

#define APP_ICON  "ClipBd"
#define APP_MENU  "ClipBd"
// note: no cursor assignment

HANDLE hInst, hMetaFile;
char   szAppTitle[] = "ClipBoard Demonstrator";
char   szAppName[]  = "ClipBd";

#define PI2 ( 2.0 * 3.1415 )

void DisplayMessage( char * Message )
{
	MessageBeep(0);
	MessageBox( GetFocus(), Message, "Attention", MB_ICONASTERISK | MB_OK );
	return;
}

BOOL TransferToClipBD( HWND hwnd, HANDLE hMemBlock, WORD FormatCB )
{
	if( OpenClipboard( hwnd ) )
	{
		EmptyClipboard();
		SetClipboardData( FormatCB, hMemBlock );
		CloseClipboard();
		return( TRUE );
	}
	return( FALSE );
}

BOOL TextToClipboard( HWND hwnd, LPSTR Txt )
{
	int           i, wLen;     
	GLOBALHANDLE  hGMem;
	LPSTR         lpGMem;

	wLen = strlen( Txt );
	hGMem = GlobalAlloc( GHND, (DWORD) wLen + 1 );
	lpGMem = GlobalLock( hGMem );
	for( i=0; i<wLen; i++ ) 
		*lpGMem++ = *Txt++;
	GlobalUnlock( hGMem );
	if( !TransferToClipBD( hwnd, hGMem, CF_TEXT ) )
		return( FALSE );
	DisplayMessage( "Text transferred to clipboard" );
	return( TRUE );
}

BOOL CreateMapMode( HDC hdc, LPMETAFILEPICT lpMFP, int cxWnd, int cyWnd )
{
	long  lMapScale;
	int   nHRes, nVRes, nHSize, nVSize;

	SetMapMode( hdc, lpMFP->mm );
	if( lpMFP->mm != MM_ISOTROPIC && lpMFP->mm != MM_ANISOTROPIC )
		return( TRUE );                     // mode's set, that's all //

	nHRes  = GetDeviceCaps( hdc, HORZRES );
	nVRes  = GetDeviceCaps( hdc, VERTRES );
	nHSize = GetDeviceCaps( hdc, HORZSIZE );
	nVSize = GetDeviceCaps( hdc, VERTSIZE );

		// positive values = suggested     //
		// image size in MM_HIMETRIC units //
	if( lpMFP->xExt > 0 )
		SetViewportExtEx( hdc, (int)((long) lpMFP->xExt * nHRes / nHSize / 100 ),
			 (int)((long) lpMFP->yExt * nHRes / nHSize / 100 ), NULL );
	else	// negative values = suggested //
			// aspect ratio, not size      //
	if( lpMFP->xExt < 0 )
	{
		lMapScale = min( ( 100L * (long) cxWnd * nHSize / nHRes / -lpMFP->xExt ),
						 ( 100L * (long) cyWnd * nVSize / nVRes / -lpMFP->yExt ) );
		SetViewportExtEx( hdc,
			(int)( (long) -lpMFP->xExt * lMapScale * nHRes / nHSize / 100 ),
			(int)( (long) -lpMFP->yExt * lMapScale * nVRes / nVSize / 100 ),
						  NULL );
	}      // zero extent = neither size //
	else   // nor aspect ratio suggested //
		SetViewportExtEx( hdc, cxWnd, cyWnd, NULL );
	return( TRUE );
}

BOOL DrawMetafile( HWND hwnd, int cxWnd, int cyWnd )
{
	LPMETAFILEPICT  lpMFP;
	GLOBALHANDLE    hGMem;
	POINT           pt[7];
	HDC             hdcMeta;
	HPEN            hPen;
	int             i, j;

		// calculate points //
	for( i=j=0; i<7; i++, j=(j+3)%7 )    // seven points //
	{
		pt[i].x = (int)( sin( j*PI2/7 ) * 100 );
		pt[i].y = (int)( cos( j*PI2/7 ) * 100 );
	}
		// create METAFILE //
	hdcMeta = CreateMetaFile( NULL );
	SetMapMode( hdcMeta, MM_ISOTROPIC );
	SetWindowExtEx( hdcMeta, 1000, 1000, NULL );
	SetViewportExtEx( hdcMeta, cxWnd, cyWnd, NULL );
	SetWindowOrgEx( hdcMeta, -300, -300, NULL );
	hPen = CreatePen( PS_NULL, 1, 0L );
	SelectObject( hdcMeta, hPen );
	SelectObject( hdcMeta, GetStockObject( LTGRAY_BRUSH ));
	Ellipse( hdcMeta, -100, -100, 100, 100 );
	SelectObject( hdcMeta, GetStockObject( DKGRAY_BRUSH ));
	SetPolyFillMode( hdcMeta, ALTERNATE );
	Polygon( hdcMeta, pt, 7 );
	hMetaFile = CloseMetaFile( hdcMeta );
	DeleteObject( hPen );
		// create METAFILEPICT //
	hGMem = GlobalAlloc( GHND, (DWORD) sizeof( METAFILEPICT ) );
	lpMFP = (LPMETAFILEPICT) GlobalLock( hGMem );
	lpMFP->mm = MM_ISOTROPIC;
	lpMFP->xExt = 200;         // suggested size in //
	lpMFP->yExt = 200;         // MM_HIMETRIC units //
	lpMFP->hMF = hMetaFile;
	GlobalUnlock( hGMem );
		// copy to clipboard //
	if( ! TransferToClipBD( hwnd, hGMem, CF_METAFILEPICT ))
		return( FALSE );
	DisplayMessage( "Metafile transferred to clipboard" );
	return( TRUE );
}

void CaptureBitmap( HWND hwnd )
{
	HDC     hdc, hdcMem;
	HBITMAP hBitmap;
	POINT   pt = { 0, 0 };

	ScreenToClient( hwnd, &pt );
	hdc = GetDC( hwnd );
	hdcMem = CreateCompatibleDC( hdc );
	hBitmap = CreateCompatibleBitmap( hdc, 640, 480 );
	if( hBitmap )
	{
		SelectObject( hdcMem, hBitmap );
		StretchBlt( hdcMem, 0,    0,    639, 479,
					hdc,    pt.x, pt.y, 639, 479, SRCCOPY );
		TransferToClipBD( hwnd, hBitmap, CF_BITMAP );
		InvalidateRect( hwnd, NULL, TRUE );
	}
	DeleteDC( hdcMem );
	ReleaseDC( hwnd, hdc );
	DisplayMessage( "Bitmap image captured to clipboard" );
	return;
}

long APIENTRY WndProc( HWND hwnd,   UINT msg,
                       UINT wParam, LONG lParam )
{
	static POINTS  pt1, pt2;
	static int     cxWnd, cyWnd, nClipRetrieve;
	char           TextStr[80];
	LPMETAFILEPICT lpMFP;
	LPSTR          lpText;
	GLOBALHANDLE   hGMem;
	PAINTSTRUCT    ps;
	BITMAP         bm;
	HBITMAP        hBitmap;
	HANDLE         hTextMem, hMenu;
	HDC            hdc, hdcMem;

	switch( msg )
	{
		case WM_CREATE:
			nClipRetrieve = 0;
			break;

		case WM_COMMAND:
			hMenu = GetMenu( hwnd );
			switch( LOWORD( wParam ) )
			{
				case IDM_PUT_BITMAP:
					CaptureBitmap( hwnd );
					EnableMenuItem( hMenu, IDM_GET_METAFILE, MF_GRAYED );
					EnableMenuItem( hMenu, IDM_GET_BITMAP, MF_ENABLED );
					EnableMenuItem( hMenu, IDM_GET_TEXT, MF_GRAYED );
					CheckMenuItem( hMenu, IDM_PUT_METAFILE,
								   MF_UNCHECKED | MF_BYCOMMAND );
					CheckMenuItem( hMenu, IDM_PUT_BITMAP,
								   MF_CHECKED | MF_BYCOMMAND );
					CheckMenuItem( hMenu, IDM_PUT_TEXT, 
								   MF_UNCHECKED | MF_BYCOMMAND );

					break;

				case IDM_PUT_METAFILE:
					DrawMetafile( hwnd, cxWnd, cyWnd );
					EnableMenuItem( hMenu, IDM_GET_METAFILE, MF_ENABLED );
					EnableMenuItem( hMenu, IDM_GET_BITMAP, MF_GRAYED );
					EnableMenuItem( hMenu, IDM_GET_TEXT, MF_GRAYED );
					CheckMenuItem( hMenu, IDM_PUT_METAFILE,
								   MF_CHECKED | MF_BYCOMMAND );
					CheckMenuItem( hMenu, IDM_PUT_BITMAP,
								   MF_UNCHECKED | MF_BYCOMMAND );
					CheckMenuItem( hMenu, IDM_PUT_TEXT, 
								   MF_UNCHECKED | MF_BYCOMMAND );
					break;

				case IDM_PUT_TEXT:
					TextToClipboard( hwnd, 
						"The quick brown fox jumped over the lazy red dog" );
					EnableMenuItem( hMenu, IDM_GET_METAFILE, MF_GRAYED );
					EnableMenuItem( hMenu, IDM_GET_BITMAP, MF_GRAYED );
					EnableMenuItem( hMenu, IDM_GET_TEXT, MF_ENABLED );
					CheckMenuItem( hMenu, IDM_PUT_METAFILE,
								   MF_UNCHECKED | MF_BYCOMMAND );
					CheckMenuItem( hMenu, IDM_PUT_BITMAP,
								   MF_UNCHECKED | MF_BYCOMMAND );
					CheckMenuItem( hMenu, IDM_PUT_TEXT, 
								   MF_CHECKED | MF_BYCOMMAND );
					break;

				case IDM_GET_METAFILE:
				case IDM_GET_BITMAP:
				case IDM_GET_TEXT:
					nClipRetrieve = wParam;
					InvalidateRect( hwnd, NULL, TRUE );
					break;

				default:
					return( DefWindowProc( hwnd, msg, wParam, lParam ) );
			}
			break;

		case WM_SIZE:
			cxWnd = LOWORD( lParam );
			cyWnd = HIWORD( lParam );
			break;

		case WM_PAINT:
			InvalidateRect( hwnd, NULL, TRUE );
			hdc = BeginPaint( hwnd, &ps );
			switch( nClipRetrieve )
			{
				case IDM_GET_TEXT:
					nClipRetrieve = 0;
					if( IsClipboardFormatAvailable( CF_TEXT ) )
					{
						OpenClipboard( hwnd );
						hTextMem = GetClipboardData( CF_TEXT );
						lpText = GlobalLock( hTextMem );
						lstrcpy( TextStr, lpText );
						GlobalUnlock( hTextMem );
						CloseClipboard();
						TextOut( hdc, 10, 10, TextStr, strlen( TextStr ) );
					}
					break;

				case IDM_GET_BITMAP:
					nClipRetrieve = 0;
					if( IsClipboardFormatAvailable( CF_BITMAP))
					{
						OpenClipboard( hwnd );
						hBitmap = GetClipboardData( CF_BITMAP );
						SetCursor( LoadCursor( NULL, IDC_WAIT));
						hdcMem = CreateCompatibleDC( hdc );
						SelectObject( hdcMem, hBitmap );
						SetMapMode( hdcMem, GetMapMode( hdc ) );
						GetObject( hBitmap, sizeof(BITMAP), (LPSTR) &bm );
						BitBlt( hdc, 0, 0, bm.bmWidth, bm.bmHeight,
								hdcMem, 0, 0, SRCCOPY );
						SetCursor( LoadCursor( NULL, IDC_ARROW ) );
						ReleaseDC( hwnd, hdc );
						DeleteDC( hdcMem );
		//======================================================//
		//  do not use DeleteObject without EmptyClipboard!!!   //
		//				DeleteObject( hBitmap );				//
		//				EmptyClipboard();						//
		//======================================================//
						CloseClipboard();
					}
					break;

				case IDM_GET_METAFILE:
					nClipRetrieve = 0;
					if( IsClipboardFormatAvailable( CF_METAFILEPICT ) )
					{
						OpenClipboard( hwnd );
						hGMem = GetClipboardData( CF_METAFILEPICT );
						lpMFP = (LPMETAFILEPICT) GlobalLock( hGMem );
						SaveDC( hdc );
						CreateMapMode( hdc, lpMFP, cxWnd, cyWnd );
						PlayMetaFile( hdc, lpMFP->hMF );
						RestoreDC( hdc, - 1 );
						GlobalUnlock( hGMem );
						CloseClipboard();
					}
					break;
			}
			EndPaint( hwnd, &ps );
			break;

		case WM_DESTROY:
			PostQuitMessage(0);
			break;

		default:	return( DefWindowProc( hwnd, msg, wParam, lParam ) );
	}
	return 0;
}

#include "template.i"

int APIENTRY WinMain( HANDLE hInstance, HANDLE hPrevInstance,
                      LPSTR  lpCmdLine, int    nCmdShow    )
{
	MSG   msg;

	if( ! hPrevInstance )
		if( ! InitApplication( hInstance ) )
			return( FALSE );
	if( ! InitInstance( hInstance, nCmdShow ) )
		return (FALSE);
	while( GetMessage( &msg, NULL, 0, 0 ) )
	{
		TranslateMessage( &msg );
		DispatchMessage( &msg );
	}
	return( msg.wParam );
	UNREFERENCED_PARAMETER( lpCmdLine );
}
